///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  FastDataSource.cpp                                                                //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//
//<LIBTAG:3655f03d-8d45-4bd6-883d-66d7ab996593>

#include  <fftw3.h>
#include  "modules.h"
#include  "Exception.h"
#include  "{VAL label}.h"


{VAL label}::{VAL label}(void) :
ColumnsRead{0},
TimerStarted(false),
NumBatchesProcessed(0),
{VAL params/out_fifo_name}_Dimensions({VAL params/fifo_out_dimensions}),
{VAL params/out_fifo_name}_WrapFlags({VAL params/fifo_out_wrap_flags}),
{VAL params/out_fifo_name}_Extent( {VAL params/out_fifo_name}_Dimensions, {VAL params/out_fifo_name}_WrapFlags ),
{VAL params/out_fifo_name}({VAL params/out_fifo_name}_Extent),
FWT({VAL params/out_fifo_name})
{
  this->State = State_Initialized;

  if( { VAL debug } )
  { // Debug display Display_Constructor
    cout << this->Display_Constructor();
  }
}


RTIF::ExitStateType  {VAL label}::Init_Allocate(void)
{
  this->State = State_Allocated;

  this->ColumnsRead = 0;

  if( { VAL debug } )
  { // Display Display_InitReset
    cout << Display_InitAlloc();
  }

  return  RTIF::Exit_Normally;
}


RTIF::ExitStateType  {VAL label}::Init_Reset(void)
{
  this->State = State_Reset;

  // Reset control variables
  this->KillFlag       = false;
  this->Alive          = 0;
  this->Complete       = 0;
  this->CycleCounter   = ~0;
  this->ColumnsRead    = 0;

  // Spawn the controller thread
  ControlThread = thread( ProcessBatches, ref(*this) );

  // See if we should spawn the optional real-time display thread
  if(this->RealtimeDisplayFlag)
  {
    RealTimeDisplayThread = thread( RealtimeDisplay, ref(*this) );
  }

  if( { VAL debug } )
  { // Display Display_InitReset
    cout << Display_InitReset();
  }

  return  RTIF::Exit_Normally;
}


RTIF::ExitStateType  {VAL label}::RealtimeDisplay( {VAL label}& _Obj )
{
  do
  {
    std::this_thread::sleep_for( {VAL params/realtime_display_update_rate_int_secs}s );

    {
      lock_guard<mutex>  Lock(_Obj.ControlMutex);
      lock_guard<mutex>  Lock2(DebugMutex);

      // Set end time
      _Obj.EndTime = chrono::high_resolution_clock::now();

      duration<double>   Elapsed(_Obj.EndTime - _Obj.StartTime);

      cout << "{VAL label}" << "[" << &_Obj << "]: "
           << setw(10) << _Obj.NumBatchesProcessed << " batches"
           << "  "
           << setw(10) << setprecision(6) << (double) _Obj.NumBatchesProcessed/Elapsed.count()
           << " batches/sec" << endl;
    }

    // Look for a Kill signal
    if( _Obj.KillFlag )  break;

  }  while(true);

  return  RTIF::Exit_Due_To_Kill;
}


RTIF::ExitStateType  {VAL label}::ProcessBatches( {VAL label}& _Obj )
{
  // State change
  _Obj.State = State_Processing;

  // Initialize the timer
  {
    lock_guard<mutex>  Lock(_Obj.ControlMutex);

    _Obj.StartTime = chrono::high_resolution_clock::now();
    _Obj.NumBatchesProcessed = 0;
    _Obj.TimerStarted = true;
  }

  do
  {
    // FIFO start and lock
    _Obj.{VAL params/out_fifo_name}.FIFO_Write_Start( {VAL params/write_size}, _Obj.FWT );

    // Look for a Kill signal
    if( _Obj.KillFlag )  return  RTIF::Exit_Due_To_Kill;

    { // Increment the cycle count to start processing
      unique_lock<mutex>   LK(_Obj.GetCvMutex);
      ++_Obj.CycleCounter;
      LK.unlock();
      _Obj.CV.notify_all();
    }

    // Look for a Kill signal
    if( _Obj.KillFlag )  return  RTIF::Exit_Due_To_Kill;

    /////////////////////////////////
    // Dummy Data
    /////////////////////////////////

    // Done writing into FIFO --- alert readers new data is available
    _Obj.{VAL params/out_fifo_name}.FIFO_Write_End(_Obj.FWT);

    // Look for a Kill signal
    if( _Obj.KillFlag )  return  RTIF::Exit_Due_To_Kill;

    // Update the sample counter
    _Obj.ColumnsRead += {VAL params/write_size};

    // Update the timer
    {
      lock_guard<mutex>  Lock(_Obj.ControlMutex);

      _Obj.EndTime = chrono::high_resolution_clock::now();
      _Obj.NumBatchesProcessed += {VAL params/write_size};
    }

  } while(true);

  return  RTIF::Exit_Normally;
}


RTIF::ExitStateType  {VAL label}::KillThreads(void)
{
  this->State = this->State_Kill;

  { // Set Kill flags in all FIFOs
    {VAL params/out_fifo_name}.SetKillFlag(true);
  }

  { // Set KillFlag in threads
    unique_lock<mutex>   LK(GetCvMutex);
    this->KillFlag = true;
    LK.unlock();
    CV.notify_all();
  }

  if( { VAL debug } )
  { // Display Display_InitReset
    cout << this->Display_Kill();
  }

  // Join up with the dying Control Thread
  this->ControlThread.join();

  // See if we should spawn the optional real-time display thread
  if(this->RealtimeDisplayFlag)
  {
    this->RealTimeDisplayThread.join();
  }

  return  RTIF::Exit_Due_To_Kill;
}


///////////////////////////////////////
// Display and Debug Routines
///////////////////////////////////////

string  {VAL label}::Display_Constructor(void)
{
  lock_guard<mutex>   LK(this->GetCvMutex);
  ostringstream       oS;

  oS << ">> {VAL label}::Display_Constructor[" << this << "]" << endl;

  // Parameters
  oS << "  -State: " << DisplayState( this->State ) << endl;

  oS << "<< {VAL label}::Display_Constructor[" << this << "]" << endl << endl;

  return  oS.str();
}


string  {VAL label}::Display_InitAlloc(void)
{
  lock_guard<mutex>   LK(this->GetCvMutex);
  ostringstream       oS;

  oS << ">> {VAL label}::Display_InitAlloc[" << this << "]" << endl;

  oS << "  -State: " << DisplayState( this->State ) << endl;

  oS << "<< {VAL label}::Display_InitAlloc[" << this << "]" << endl << endl;

  return    oS.str();
}


string  {VAL label}::Display_InitReset(void)
{
  lock_guard<mutex>   LK(this->GetCvMutex);
  ostringstream       oS;

  oS << ">> {VAL label}::Display_InitReset[" << this << "]" << endl;

  oS << "  -State: " << DisplayState( this->State ) << endl;

  oS << "<< {VAL label}::Display_InitReset[" << this << "]" << endl << endl;

  return    oS.str();
}


string  {VAL label}::Display_Kill(void)
{
  lock_guard<mutex>   LK(this->GetCvMutex);
  ostringstream       oS;

  oS << ">> {VAL label}::Display_Kill[" << this << "]" << endl;

  oS << "  -KillFlag: " << boolalpha << this->KillFlag << endl;
  oS << "  -CycleCounter: " << this->CycleCounter << endl;
  oS << "  -State: " << DisplayState( this->State ) << endl;

  oS << "<< {VAL label}::Display_Kill[" << this << "]" << endl << endl;

  return    oS.str();
}


