///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  AndRuleObj.cpp                                                                    //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include        <cassert>
#include        <typeinfo>
#include        <iostream>
#include        <iomanip>
#include        <sstream>
#include        <typeinfo>
#include        "AndRuleObj.h"
#include        "OrRuleObj.h"
#include        "CharacterObj.h"
#include        "BNFC_Gen.h"
#include        "KleinWrapperObj.h"
#include        "RefObj.h"
#include        "LiteralObj.h"
#include        "OutputListObj.h"
#include        "GrammarObj.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
// AndRuleObj::SpaceBetweenRules
///////////////////////////////////////////////////////////////////////////////////////////////////

const char*      AndRuleObj::SpaceBetweenRules = "  ";


///////////////////////////////////////////////////////////////////////////////////////////////////
// void      AndRuleObj::PushBack
///////////////////////////////////////////////////////////////////////////////////////////////////

void      AndRuleObj::PushBack( GrammarAbs* Rule )
{
  Group.push_back(Rule);
  Rule->SetFather(this);

  // Locate the father OR rule
  GrammarAbs*     P;

  // Scan upwards for OR rule
  for( P=this->GetFather() ; P && (typeid(*P) != typeid(OrRuleObj)) ; P = P->GetFather() );

  // If we found it save it
  FatherOrGroup   = dynamic_cast<OrRuleObj*>(P);

  assert(FatherOrGroup);
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// AndRuleObj::~AndRuleObj
///////////////////////////////////////////////////////////////////////////////////////////////////

AndRuleObj::~AndRuleObj()
{
  unsigned int    K;

  for( K=0 ; K < Group.size() ; ++K )
  {
    delete  Group[K];
  }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// string      AndRuleObj::Display
///////////////////////////////////////////////////////////////////////////////////////////////////

string      AndRuleObj::Display( bool SubRule, const unsigned int Indent ) const
{
  unsigned int              K;
  ostringstream             oS;
  bool                      First;

  if(SubRule)   oS << SpaceBetweenRules << "(";

  for( K=0, First=true ; K < Group.size() ; ++K, First=false )
  {
    if(!First)  oS << SpaceBetweenRules;
    oS << Group[K]->Display(true,Indent+2);
  }

  oS << SpaceBetweenRules;

  if(SubRule)   oS << ")" << SpaceBetweenRules;

  oS << DisplayFlags();

  return    oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// bool      AndRuleObj::Parse
///////////////////////////////////////////////////////////////////////////////////////////////////

bool      AndRuleObj::Parse( ParseFrame PF )
{
  unsigned int      K;
  bool              R;

  // Set rule
  PF.SetRule(this);

  // Save the current file position
  PF.SaveInputFilePosition(0);

  for( K=0, R=true ; K < Group.size() ; ++K )
  {
    // Parse the Normal Rule
    R   = Group[K]->Parse(PF);

    // No match
    if(!R)    break;

    // Save the current file position
    PF.SaveInputFilePosition(K+1);
  }

  bool    Good  = R && ( K == Group.size() );

  // Accept the data on the streams
  if(Good)    PF.AcceptAll();

  if(GrammarAbs::DebugParse)
  {
    cout << ">>AND"
         << "  " << (R ? "<++++++>" : "<------>")
         << "  [" << PF.GetRuleDebugDescriptor() << "]"
         << "  [" << PF.GetInPos() << "," << PF.GetOutListPos() << "]"
         << endl;
  }

  return    Good;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// string     AndRuleObj::GetOutput
///////////////////////////////////////////////////////////////////////////////////////////////////

string     AndRuleObj::GetOutput( ParseFrame& PF, const unsigned int RuleNum )
{
  UNUSED(PF);
  UNUSED(RuleNum);

  return    string();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// void      AndRuleObj::GenerateCppParseTree
///////////////////////////////////////////////////////////////////////////////////////////////////

void      AndRuleObj::GenerateCppParseTree( IBnfToCppCov& OutputData, bool KleinLayer, string& Includes )
{
  unsigned int        K;

  for( K=0 ; K < Group.size() ; ++K )
  {
    ostringstream     oS;

    oS << endl;
    oS << "  // RULE #" << (K+1) << ": '" << Group[K]->Display() << "'" << endl;

    // Accumulate the output
    OutputData.Subroutines.back().Contents  += oS.str();

    // Get lower-level constructs
    Group[K]->GenerateCppParseTree( OutputData, KleinLayer, Includes );
  }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// bool      AndRuleObj::InverseParse
///////////////////////////////////////////////////////////////////////////////////////////////////

bool      AndRuleObj::InverseParse( InverseParseFrame IPF )
{
  // Set rule
  IPF.SetRule(this);

  assert(!Group.empty());

  // If Debug
  if( IPF.IsDebug() )
  {
    cout << ">> IParse AND[" << IPF.Display() << "]" << endl;
  }

  // Over the rules in the group
  for( auto S: Group )
  {
    // Parse using the first rule
    bool    R = S->InverseParse(IPF);

    // If we didn't pass we fail and return
    if( !R )
    {
      if( IPF.IsDebug() )
      {
        cout << "<<-IParse AND[" << IPF.Display() << "]" << endl;
      }

     return  false;
    }
  }

  // If we passed
  IPF.AcceptAll();

  if( IPF.IsDebug() )
  {
    cout << "<<+IParse AND[" << IPF.Display() << "]" << endl;
  }

  return  true;
}
}   // End namespace 'wej_parser'


