///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  DataPool.cpp                                                                      //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include  "DataPool.h"
#include  "Exception.h"


DataPool::NodeRefType    DataPool::FindCreatePath( const DataPath& _Path )
{
  NodeRefType   NRT;

  for( auto I: DataSources )
  {
    auto  A  = I->FindCreatePath( _Path );

    if( A == nullptr )  continue;

    NRT.Tree  = &*I;
    NRT.Node  = A;

    break;
  }

  return    NRT;
}


DataPool::NodeRefType    DataPool::FindPath( const DataPath& _Path )
{
  NodeRefType   NRT;

  for( auto I: DataSources )
  {
    auto  A  = I->FindPath( _Path );

    if( A == nullptr )  continue;

    NRT.Tree  = &*I;
    NRT.Node  = A;

    break;
  }

  return    NRT;
}


void    DataPool::Write( const DataPool::NodeRefType& _NodeRef, const string& _DataIn )
{
  if( !_NodeRef.IsValid() )
  {
    Exception   DE(__FILE__,__LINE__);
    DE.Msg() << "DataPool::Write: Called with an invalid _NodeRef."<< endl;
    throw   DE;
  }

  // Write the data
  _NodeRef.Node->SetData(_DataIn);
}


string  DataPool::Read ( const DataPool::NodeRefType& _NodeRef )
{
  if( !_NodeRef.IsValid() )
  {
    Exception   DE(__FILE__,__LINE__);
    DE.Msg() << "DataPool::Read: Called with an invalid _NodeRef."<< endl;
    throw   DE;
  }

  // Get the data
  return    _NodeRef.Node->GetData();
}


void    DataPool::WriteTreeToDisk( DataTree* _Tree )
{
  if( _Tree == nullptr )
  {
    Exception   DE(__FILE__,__LINE__);
    DE.Msg() << "DataPool::WriteTreeToDisk: Called with a a 'nullptr' tree."<< endl;
    throw   DE;
  }

  _Tree->WriteFileToDisk();
}


///////////////////////////////////////////////////////////////////////////////
// Global Data Pool
///////////////////////////////////////////////////////////////////////////////

namespace   wej_data_pool
{
  DataPool    DataTrees;
}


