///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  DataTree.h                                                                        //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include  <string>
#include  <mutex>
#include  "XML_Storage.h"
#include  "DataPath.h"

using namespace std;


class     DataTree
{
public:

  typedef enum
  {
    eMODE_LOAD_FROM_FILE,
    eMODE_SAVE_TO_FILE,
    eMODE_LOAD_AND_SAVE_TO_FILE,
  }   CD_ModesType;

private:

  // Parameters
  CD_ModesType      Mode;           // Read/Write from files modes
  string            FilePath;       // Path to disc file
  DataPath          DataPrefix;     // Prefix to this data tree

  wej_parser::XML_Storage   XML;    // Storage in memory

public:

  DataTree( const CD_ModesType  _Mode, const DataPath _DataPrefix=DataPath() );
  DataTree( const string& _FilePath, const CD_ModesType  _Mode, const DataPath _DataPrefix=DataPath() );
  ~DataTree(void);

  // 'nullptr' returned if cannot be found or other error
  wej_parser::XML_Node*   FindCreatePath( DataPath _Path );
  wej_parser::XML_Node*   FindPath( DataPath _Path );

  inline void      Write( wej_parser::XML_Node* _Node, const string& _Data )
  {
    assert(_Node);
    _Node->SetData(_Data);
  }

  inline string    Read ( wej_parser::XML_Node* _Node )
  {
    assert(_Node);
    return  _Node->GetData();
  }

  // Force write of tree to the disk
  inline void      WriteFileToDisk(void)
  {
    XML.WriteToXML_File(FilePath);
  }

  inline string    DisplayTree( const unsigned int _Indent=0 ) const
  {
    return    XML.GetHead().Display(_Indent);
  }
};


