///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  GrammarAbs.h                                                                      //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include    <sstream>
#include    <string>
#include    "ParseFrame.h"
#include    "InverseParseFrame.h"
#include    "IBnfToCppCov.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
// class     GrammarAbs
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class     AndRuleObj;

class     GrammarAbs
{
public:

#define UNUSED(x) (void)(x)

  // Control
  static const bool         DebugParse = false;

  // Input text stream
  typedef    unsigned char  CharType;

  // Internal char representation
  typedef    unsigned int   IntCharType;

  // Modifiers
  typedef enum eKlienFlags
  {
    NoKlien,
    KlienStar,
    KlienPlus,
    KlienQuestion
  }    KlienFlagsType;

private:

  string        Label;
  GrammarAbs*   Father;

public:

  inline GrammarAbs(void) : Father(nullptr)
  {
  }

  inline virtual ~GrammarAbs(void)
  {
  }

  inline GrammarAbs*    GetFather(void) const
  {
    return  Father;
  }

  inline void           SetFather( GrammarAbs* pFather )
  {
    Father  = pFather;
  }

  inline string         GetLabel(void) const
  {
    return  Label;
  }

  inline void           SetLabel( const string& S )
  {
    Label   = S;
  }

  inline string         DisplayFlags(void) const
  {
    return  string();
  }

  //////////////////////////////
  // Interfaces
  //////////////////////////////

  virtual string      Display( bool SubRule=false, const unsigned int Inden=0 ) const = 0;
  virtual bool        Parse( ParseFrame PF ) = 0;
  virtual string      GetOutput( ParseFrame& PF, const unsigned int RuleNum ) = 0;
  virtual void        GenerateCppParseTree( IBnfToCppCov& OutputData, bool KleinLayer, string& Includes ) = 0;
  virtual bool        DoesRead(void) const = 0;
  virtual bool        InverseParse( InverseParseFrame IPF ) = 0;
};
}   // End namespace 'wej_parser'


