///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  IBnfToCppCov.cpp                                                                  //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include  <iostream>
#include  <sstream>
#include  <iomanip>
#include  "IBnfToCppCov.h"
#include  "BNFC_Gen.h"
#include  "GrammarAbs.h"
#include  "Translator.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
// IBnfToCppCov::IBnfToCppCov
///////////////////////////////////////////////////////////////////////////////////////////////////

IBnfToCppCov::IBnfToCppCov( istream& InBnfGrammar )
{
  HC  = Translator::GetIBnfFromHardCodedBnfc( InBnfGrammar );
  assert(HC);

  // Generate the C++ code tree
  HC->GenerateCppParseTree( *this, false, Includes );
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// IBnfToCppCov::~IBnfToCppCov
///////////////////////////////////////////////////////////////////////////////////////////////////

IBnfToCppCov::~IBnfToCppCov(void)
{
  if(HC)  delete  HC;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// string    IBnfToCppCov::GenerateCppFile
///////////////////////////////////////////////////////////////////////////////////////////////////

string    IBnfToCppCov::GenerateCppFile( const string& Namespace, const string& HeaderFileName, const string& Namespace2 )
{
  ostringstream                           oS;
  list<SubRoutineType>::const_iterator    I;

  oS   << endl;
  oS   << "#include        <cassert>" << endl;
  oS   << "#include        <iostream>" << endl;
  oS   << "#include        <iomanip>" << endl;
  oS   << "#include        <sstream>" << endl;
  oS   << "#include        <typeinfo>" << endl;
  oS   << "#include        \"GrammarAbs.h\"" << endl;
  oS   << "#include        \"GrammarObj.h\"" << endl;
  oS   << "#include        \"CharacterObj.h\"" << endl;
  oS   << "#include        \"BNFC_Gen.h\"" << endl;
  oS   << "#include        \"KleinWrapperObj.h\"" << endl;
  oS   << "#include        \"RefObj.h\"" << endl;
  oS   << "#include        \"LiteralObj.h\"" << endl;
  oS   << "#include        \"OutputListObj.h\"" << endl;
  oS   << "#include        \"AndRuleObj.h\"" << endl;
  oS   << "#include        \"OrRuleObj.h\"" << endl;
  oS   << "#include        \"" << HeaderFileName << "\"" << endl << endl;
  oS   << "using namespace std;" << endl;
  oS   << "using namespace wej_parser;" << endl;
  oS   << endl << endl;

  oS << "namespace " << Namespace << endl << "{" << endl;

  if(Namespace2.length() != 0)    oS << "namespace " << Namespace2 << endl << "{" << endl;

  // Generate Outputs
  for( I=Subroutines.begin() ; I != Subroutines.end() ; ++I )
  {
    oS << I->Contents;
  }

  oS << endl << "}" << endl;

  if(Namespace2.length() != 0)    oS << "}" << endl;

  oS << endl;

  return    oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// string    IBnfToCppCov::GenerateHppFile
///////////////////////////////////////////////////////////////////////////////////////////////////

string    IBnfToCppCov::GenerateHppFile( const string& Namespace, const string& Namespace2 )
{
  ostringstream     oS;

  oS << endl;
  oS << "#include  \"GrammarAbs.h\"" << endl << endl  << endl;

  oS << "namespace " << Namespace << endl << "{" << endl;

  if(Namespace2.length() != 0)    oS << "namespace " << Namespace2 << endl << "{" << endl;

  oS << "  // Main Parser Entry Point" << endl;
  oS << "  wej_parser::GrammarAbs*    Parse_" << HC->GetLabel() << "(void);" << endl;

  oS << "}" << endl << endl;

  if(Namespace2.length() != 0)    oS << "}" << endl;

  return    oS.str();
}
}   // End namespace 'wej_parser'


