///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  OutputListObj.cpp                                                                 //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include    <cassert>
#include    <iostream>
#include    <sstream>
#include    <iomanip>
#include    <typeinfo>
#include    "OutputListObj.h"
#include    "ParseFrame.h"
#include    "LiteralObj.h"
#include      "AndRuleObj.h"

using namespace std;


namespace  wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
// string   OutputListObj::Display
///////////////////////////////////////////////////////////////////////////////////////////////////


// WEJTAG FIX ME   !!!!!

string    OutputListObj::Display( bool , const unsigned int  ) const
{
  ostringstream   oS;

  switch(EnumType)
  {
    case  NewEntry:
      oS << "$(" << OutputLabel;
    break;

    case  AppendItem:
      oS << "$>(";
    break;

    case  AppendChar:
      oS << "$>>(";
    break;

    default:
      assert(false);
    break;
  }

  for( auto I = RuleNumList.begin() ; I != RuleNumList.end() ; ++I )
  {
    if( EnumType == NewEntry )    oS << ",";
    oS << I->GetRuleNum();
  }

  oS << ")";
  oS << DisplayFlags();

  return    oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// bool   OutputListObj::Parse
///////////////////////////////////////////////////////////////////////////////////////////////////

bool    OutputListObj::Parse( ParseFrame PF )
{
  list<StatementRef>::iterator    I;
  ostringstream                   oS;

  // Set rule
  PF.SetRule(this);

  switch(EnumType)
  {
    case  NewEntry:
    {
      // To output list
      PF.PushNewOutListRecord();

      // Write the label record
      PF.EndOutListRecStrings().push_back(OutputLabel);

      // Write the Output to the output streams
      for ( I = RuleNumList.begin() ; I != RuleNumList.end() ; ++I )
      { // Push the output string to the end of the output list
        PF.EndOutListRecStrings().push_back( I->GetOutput( PF, I->GetRuleNum() ) );
      }
    }
    break;

    case  AppendItem:
    {
      assert( PF.GetOutListPos() > 0 );

      // Write the Output to the output streams
      for ( I = RuleNumList.begin() ; I != RuleNumList.end() ; ++I )
      {
        PF.EndOutListRecStrings().push_back( I->GetOutput( PF, I->GetRuleNum() ) );
      }
    }
    break;

    case  AppendChar:
    {
      assert( PF.GetOutListPos() > 0 );
      assert( PF.EndOutListRecStrings().size() > 0 );

      // Write the Output to the output streams
      for ( I = RuleNumList.begin() ; I != RuleNumList.end() ; ++I )
      {
        PF.EndOutListRecStrings().back() +=  I->GetOutput( PF, I->GetRuleNum() );
      }
    }
    break;

    default:
    {
      assert(false);
    }
    break;
  }

  PF.AcceptAll();

  // Cache the output in case another output command references this one
  CachedOutput  = PF.EndOutListRec();

  if(GrammarAbs::DebugParse)
  {
    cout  << ">>OUT_LIST  "
          << "\'" << oS.str() << "\'  "
          << this->Display(false) << "  "
          << "[" << PF.GetRuleDebugDescriptor() << "]  "
          << "[" << PF.GetInPos() << "," << PF.GetOutListPos() << "]"
          << endl;
  }

  return    true;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// void   OutputListObj::GenerateCppParseTree
///////////////////////////////////////////////////////////////////////////////////////////////////

void  OutputListObj::GenerateCppParseTree( IBnfToCppCov& OutputData, bool KleinLayer, string& Includes )
{
  ostringstream     oS;
  UNUSED(Includes);

  switch(EnumType)
  {
    case  NewEntry:
      oS << "  OUTL = new OutputListObj(OutputListObj::NewEntry);" << endl;
    break;

    case  AppendItem:
      oS << "  OUTL = new OutputListObj(OutputListObj::AppendItem);" << endl;
    break;

    case  AppendChar:
      oS << "  OUTL = new OutputListObj(OutputListObj::AppendChar);" << endl;
    break;

    default:
      assert(false);
    break;
  }

  if( KleinLayer )
  {
    oS << "  KLEIN->SetKleinChild(OUTL);" << endl;
  }
  else
  {
    oS << "  AND->PushBack(OUTL);" << endl;
  }

  switch(EnumType)
  {
    case  NewEntry:
      oS << "  OUTL->SetLabel(\"" << OutputLabel << "\");" << endl;
    break;

    case  AppendItem:
    case  AppendChar:
      // Nothing
    break;

    default:
      assert(false);
    break;
  }

  for( auto I = RuleNumList.begin() ; I != RuleNumList.end() ; ++I )
  {
    oS << "  OUTL->PushRuleNumBack(" << I->GetRuleNum() << ");" << endl;
  }

  // Accumulate the output
  OutputData.Subroutines.back().Contents  += oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// bool   OutputListObj::InverseParse
///////////////////////////////////////////////////////////////////////////////////////////////////

bool  OutputListObj::InverseParse( InverseParseFrame IPF )
{
  // Set rule
  IPF.SetRule(this);

  if( IPF.IsDebug() )
  {
    cout << ">> IParse OUT[" << IPF.Display() << "]" << endl;
  }

  // Write the Output to the output streams
  for ( auto I = this->RuleNumList.begin() ; I != this->RuleNumList.end() ; ++I )
  {
    IPF.WriteStringToOutput( IPF.GetListPos()->GetDataVect()[ I->GetRuleNum() ] );
  }

  // Pass by default
  IPF.AcceptAll();

  if( IPF.IsDebug() )
  {
    cout << "<<+IParse OUT[" << IPF.Display() << "]" << endl;
  }

  return    true;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// OutputListObj::StatementRef::GetOutput( ParseFrame& PF, const unsigned int RuleNum )
///////////////////////////////////////////////////////////////////////////////////////////////////

string    OutputListObj::StatementRef::GetOutput( ParseFrame& PF, const unsigned int RuleNum )
{
  string        Ret;
  ParseFrame*   PF_AtAnd;

  // Get pointers to the rule and the associated parse frame
  auto  AndRule = PF.FindUpperAndRule( &PF_AtAnd, this->RuleName );

  // Make sure we found the statement
  assert( AndRule );

  // This is the 'AndRuleObj'
  GrammarAbs*   GA = AndRule->GetRule( this->RuleNum );
  assert(GA);

  // Recover the output string
  Ret = GA->GetOutput( *PF_AtAnd, RuleNum );

  // Recover the output
  return    Ret;
}
}   // End namespace 'wej_parser'


