///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  XML_Storage_Gram.cpp                                                              //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include        <cassert>
#include        <iostream>
#include        <iomanip>
#include        <sstream>
#include        <typeinfo>
#include        "GrammarAbs.h"
#include        "GrammarObj.h"
#include        "CharacterObj.h"
#include        "BNFC_Gen.h"
#include        "KleinWrapperObj.h"
#include        "RefObj.h"
#include        "LiteralObj.h"
#include        "OutputListObj.h"
#include        "AndRuleObj.h"
#include        "OrRuleObj.h"
#include        "XML_Storage_Gram.h"

using namespace std;
using namespace wej_parser;


namespace wej_parser
{
namespace wej_xml_parser
{
// Variables used in subroutines
GrammarObj*       GRAM;
CharacterObj*     CHAR;
KleinWrapperObj*  KLEIN;
RefObj*           REF;
LiteralObj*       LIT;
OutputListObj*    OUTL;
AndRuleObj*       AND;
OrRuleObj*        OR;

// Function Prototypes
GrammarAbs*    Parse_Start(void);
GrammarAbs*    Parse_XML_Dec(void);
GrammarAbs*    Parse_Node(void);
GrammarAbs*    Parse_NodeEmpty(void);
GrammarAbs*    Parse_NodeStart(void);
GrammarAbs*    Parse_NodeEnd(void);
GrammarAbs*    Parse_NodeInnerd(void);
GrammarAbs*    Parse_NodeLabels(void);
GrammarAbs*    Parse_CNode(void);
GrammarAbs*    Parse_CNodeInner(void);
GrammarAbs*    Parse_CNodeEnd(void);
GrammarAbs*    Parse_String(void);
GrammarAbs*    Parse_StringInner(void);
GrammarAbs*    Parse_NodeData(void);
GrammarAbs*    Parse_DataChars(void);
GrammarAbs*    Parse_Identifier(void);
GrammarAbs*    Parse_CComment(void);
GrammarAbs*    Parse_CCommentInside(void);
GrammarAbs*    Parse_CppComment(void);
GrammarAbs*    Parse_WSO(void);
GrammarAbs*    Parse_WS(void);
GrammarAbs*    Parse_WS_Element(void);

// XEQ Includes


GrammarAbs*   Parse_XML(void)
{
  ///////////////////////////////////////////////////////////
  // GRAM Group
  ///////////////////////////////////////////////////////////

  GRAM  = new GrammarObj;
  GRAM->SetLabel("XML");

  ///////////////////////////////////////////////////////////
  // Rule Entry Points
  ///////////////////////////////////////////////////////////

  GRAM->PushBack(Parse_Start());
  GRAM->PushBack(Parse_XML_Dec());
  GRAM->PushBack(Parse_Node());
  GRAM->PushBack(Parse_NodeEmpty());
  GRAM->PushBack(Parse_NodeStart());
  GRAM->PushBack(Parse_NodeEnd());
  GRAM->PushBack(Parse_NodeInnerd());
  GRAM->PushBack(Parse_NodeLabels());
  GRAM->PushBack(Parse_CNode());
  GRAM->PushBack(Parse_CNodeInner());
  GRAM->PushBack(Parse_CNodeEnd());
  GRAM->PushBack(Parse_String());
  GRAM->PushBack(Parse_StringInner());
  GRAM->PushBack(Parse_NodeData());
  GRAM->PushBack(Parse_DataChars());
  GRAM->PushBack(Parse_Identifier());
  GRAM->PushBack(Parse_CComment());
  GRAM->PushBack(Parse_CCommentInside());
  GRAM->PushBack(Parse_CppComment());
  GRAM->PushBack(Parse_WSO());
  GRAM->PushBack(Parse_WS());
  GRAM->PushBack(Parse_WS_Element());

  return  GRAM;
}


GrammarAbs*    Parse_Start(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Start'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Start");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(XML_Dec)'
  REF = new RefObj;
  REF->SetLink("XML_Dec");
  AND->PushBack(REF);

  // RULE #2: 'REF(CNode)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("CNode");
  KLEIN->SetKleinChild(REF);

  // RULE #3: 'REF(Node)'
  REF = new RefObj;
  REF->SetLink("Node");
  AND->PushBack(REF);

  // RULE #4: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #5: 'CHAR(EOF)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 256 );

  return    OR;
}


GrammarAbs*    Parse_XML_Dec(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'XML_Dec'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("XML_Dec");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(<)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '<' );

  // RULE #2: 'CHAR(?)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '?' );

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'CHAR(x)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'x' );

  // RULE #5: 'CHAR(m)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'm' );

  // RULE #6: 'CHAR(l)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'l' );

  // RULE #7: '$(XML_DEC)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("XML_DEC");

  // RULE #8: 'REF(NodeLabels)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("NodeLabels");
  KLEIN->SetKleinChild(REF);

  // RULE #9: 'CHAR(?)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '?' );

  // RULE #10: 'CHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );

  return    OR;
}


GrammarAbs*    Parse_Node(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Node'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Node");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(NodeEmpty)'
  REF = new RefObj;
  REF->SetLink("NodeEmpty");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(NodeStart)'
  REF = new RefObj;
  REF->SetLink("NodeStart");
  AND->PushBack(REF);

  // RULE #2: 'REF(NodeInnerd)'
  REF = new RefObj;
  REF->SetLink("NodeInnerd");
  AND->PushBack(REF);

  // RULE #3: 'REF(NodeEnd)'
  REF = new RefObj;
  REF->SetLink("NodeEnd");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_NodeEmpty(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'NodeEmpty'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("NodeEmpty");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(<)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '<' );

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #5: '$(NODE_EMPTY,4)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("NODE_EMPTY");
  OUTL->PushRuleNumBack(4);

  // RULE #6: 'REF(NodeLabels)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("NodeLabels");
  KLEIN->SetKleinChild(REF);

  // RULE #7: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #8: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #9: 'CHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );

  // RULE #10: '$(NODE_EMPTY_DONE)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("NODE_EMPTY_DONE");

  return    OR;
}


GrammarAbs*    Parse_NodeStart(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'NodeStart'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("NodeStart");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(<)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '<' );

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #5: '$(NODE_START,4)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("NODE_START");
  OUTL->PushRuleNumBack(4);

  // RULE #6: 'REF(NodeLabels)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("NodeLabels");
  KLEIN->SetKleinChild(REF);

  // RULE #7: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #8: 'CHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );

  return    OR;
}


GrammarAbs*    Parse_NodeEnd(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'NodeEnd'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("NodeEnd");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(<)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '<' );

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #5: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #6: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #7: '$(NODE_END,6)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("NODE_END");
  OUTL->PushRuleNumBack(6);

  // RULE #8: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #9: 'CHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );

  return    OR;
}


GrammarAbs*    Parse_NodeInnerd(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'NodeInnerd'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("NodeInnerd");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(Node)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("Node");
  KLEIN->SetKleinChild(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(NodeData)'
  REF = new RefObj;
  REF->SetLink("NodeData");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CNode)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("CNode");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_NodeLabels(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'NodeLabels'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("NodeLabels");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WS_Element)'
  REF = new RefObj;
  REF->SetLink("WS_Element");
  AND->PushBack(REF);

  // RULE #2: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'CHAR(=)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '=' );

  // RULE #5: '$(NODE_LABEL,2)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("NODE_LABEL");
  OUTL->PushRuleNumBack(2);

  // RULE #6: '$(NODE_STRING)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("NODE_STRING");

  // RULE #7: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #8: 'REF(String)'
  REF = new RefObj;
  REF->SetLink("String");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_CNode(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CNode'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CNode");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(<)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '<' );

  // RULE #3: 'CHAR(!)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '!' );

  // RULE #4: 'CHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );

  // RULE #5: 'CHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );

  // RULE #6: 'REF(CNodeInner)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("CNodeInner");
  KLEIN->SetKleinChild(REF);

  // RULE #7: 'REF(CNodeEnd)'
  REF = new RefObj;
  REF->SetLink("CNodeEnd");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_CNodeInner(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CNodeInner'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CNodeInner");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'NCHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );
  CHAR->Invert();

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );

  // RULE #2: 'NCHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );
  CHAR->Invert();

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );

  // RULE #2: 'CHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );

  // RULE #3: 'NCHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );
  CHAR->Invert();

  return    OR;
}


GrammarAbs*    Parse_CNodeEnd(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CNodeEnd'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CNodeEnd");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );

  // RULE #2: 'CHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );

  // RULE #3: 'CHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );

  return    OR;
}


GrammarAbs*    Parse_String(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'String'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("String");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(")'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '"' );

  // RULE #2: 'REF(StringInner)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("StringInner");
  KLEIN->SetKleinChild(REF);

  // RULE #3: 'CHAR(")'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '"' );

  // RULE #4: '$>(2)'
  OUTL = new OutputListObj(OutputListObj::AppendItem);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(2);

  return    OR;
}


GrammarAbs*    Parse_StringInner(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'StringInner'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("StringInner");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'NCHAR(")'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '"' );
  CHAR->Invert();

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\\)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\\' );

  // RULE #2: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  return    OR;
}


GrammarAbs*    Parse_NodeData(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'NodeData'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("NodeData");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'LIT()'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("");

  // RULE #2: '$(NODE_DATA,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("NODE_DATA");
  OUTL->PushRuleNumBack(1);

  // RULE #3: 'REF(DataChars)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("DataChars");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_DataChars(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'DataChars'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("DataChars");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(&)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '&' );

  // RULE #2: 'CHAR(l)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'l' );

  // RULE #3: 'CHAR(t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 't' );

  // RULE #4: 'CHAR(;)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ';' );

  // RULE #5: 'LIT(<)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("<");

  // RULE #6: '$>>(5)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(5);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(&)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '&' );

  // RULE #2: 'CHAR(g)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'g' );

  // RULE #3: 'CHAR(t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 't' );

  // RULE #4: 'CHAR(;)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ';' );

  // RULE #5: 'LIT(>)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue(">");

  // RULE #6: '$>>(5)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(5);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(&)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '&' );

  // RULE #2: 'CHAR(a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'a' );

  // RULE #3: 'CHAR(m)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'm' );

  // RULE #4: 'CHAR(p)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'p' );

  // RULE #5: 'CHAR(;)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ';' );

  // RULE #6: 'LIT(&)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("&");

  // RULE #7: '$>>(6)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(6);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(&)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '&' );

  // RULE #2: 'CHAR(a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'a' );

  // RULE #3: 'CHAR(p)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'p' );

  // RULE #4: 'CHAR(o)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'o' );

  // RULE #5: 'CHAR(s)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 's' );

  // RULE #6: 'CHAR(;)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ';' );

  // RULE #7: 'LIT(')'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("\'");

  // RULE #8: '$>>(7)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(7);

  ///////////////////////////////////////////////////////////
  // AND Group #4
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(&)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '&' );

  // RULE #2: 'CHAR(q)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'q' );

  // RULE #3: 'CHAR(u)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'u' );

  // RULE #4: 'CHAR(o)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'o' );

  // RULE #5: 'CHAR(t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 't' );

  // RULE #6: 'CHAR(;)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ';' );

  // RULE #7: 'LIT(")'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("\"");

  // RULE #8: '$>>(7)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(7);

  ///////////////////////////////////////////////////////////
  // AND Group #5
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'NCHAR(<)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '<' );
  CHAR->Invert();

  // RULE #2: '$>>(1)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(1);

  return    OR;
}


GrammarAbs*    Parse_Identifier(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Identifier'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Identifier");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(:,A-Z,_,a-z)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ':' );
  CHAR->SetRangeChars( 'A', 'Z' );
  CHAR->SetChar( '_' );
  CHAR->SetRangeChars( 'a', 'z' );

  // RULE #2: 'CHAR(--.,0-:,A-Z,_,a-z)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '-', '.' );
  CHAR->SetRangeChars( '0', ':' );
  CHAR->SetRangeChars( 'A', 'Z' );
  CHAR->SetChar( '_' );
  CHAR->SetRangeChars( 'a', 'z' );

  return    OR;
}


GrammarAbs*    Parse_CComment(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CComment'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CComment");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #2: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  // RULE #3: 'REF(CCommentInside)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("CCommentInside");
  KLEIN->SetKleinChild(REF);

  // RULE #4: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  // RULE #5: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  return    OR;
}


GrammarAbs*    Parse_CCommentInside(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CCommentInside'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CCommentInside");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  // RULE #2: 'NCHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );
  CHAR->Invert();

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'NCHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );
  CHAR->Invert();

  return    OR;
}


GrammarAbs*    Parse_CppComment(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CppComment'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CppComment");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #2: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #3: 'NCHAR(\x0a)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetChar( '\x0a' );
  CHAR->Invert();

  // RULE #4: 'CHAR(\x0a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\x0a' );

  return    OR;
}


GrammarAbs*    Parse_WSO(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WSO'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WSO");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WS_Element)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("WS_Element");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_WS(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WS'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WS");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WS_Element)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("WS_Element");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_WS_Element(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WS_Element'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WS_Element");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\x09-\x0a,\x0d,\x20)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '\x09', '\x0a' );
  CHAR->SetChar( '\x0d' );
  CHAR->SetChar( '\x20' );

  return    OR;
}



}
}

