///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  BNFC_Gen.cpp                                                                      //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include        <cassert>
#include        <iostream>
#include        <iomanip>
#include        <sstream>
#include        <typeinfo>
#include        "GrammarAbs.h"
#include        "GrammarObj.h"
#include        "CharacterObj.h"
#include        "BNFC_Gen.h"
#include        "KleinWrapperObj.h"
#include        "RefObj.h"
#include        "LiteralObj.h"
#include        "OutputListObj.h"
#include        "AndRuleObj.h"
#include        "OrRuleObj.h"
#include        "BNFC_Gen.h"

using namespace std;
using namespace wej_parser;


namespace wej_parser
{
namespace wej_bnf
{
// Variables used in subroutines
GrammarObj*       GRAM;
CharacterObj*     CHAR;
KleinWrapperObj*  KLEIN;
RefObj*           REF;
LiteralObj*       LIT;
OutputListObj*    OUTL;
AndRuleObj*       AND;
OrRuleObj*        OR;

// Function Prototypes
GrammarAbs*    Parse_Start(void);
GrammarAbs*    Parse_ParserLabel(void);
GrammarAbs*    Parse_PumpLabel(void);
GrammarAbs*    Parse_Statement(void);
GrammarAbs*    Parse_OrStatement(void);
GrammarAbs*    Parse_OrStatement2(void);
GrammarAbs*    Parse_AndStatement(void);
GrammarAbs*    Parse_AndCommand2(void);
GrammarAbs*    Parse_Command(void);
GrammarAbs*    Parse_CommandType(void);
GrammarAbs*    Parse_OutputListCommandAppendItem(void);
GrammarAbs*    Parse_OutputListCommandAppendChar(void);
GrammarAbs*    Parse_OutputListCommand(void);
GrammarAbs*    Parse_OutputListCommand2(void);
GrammarAbs*    Parse_OutputListStatementLabel(void);
GrammarAbs*    Parse_OutputListStmntRef(void);
GrammarAbs*    Parse_RefCommand(void);
GrammarAbs*    Parse_REF_Lit(void);
GrammarAbs*    Parse_LiteralCommand(void);
GrammarAbs*    Parse_LIT_Lit(void);
GrammarAbs*    Parse_LiteralElement(void);
GrammarAbs*    Parse_LiteralEscapeChar(void);
GrammarAbs*    Parse_HexChar(void);
GrammarAbs*    Parse_TagCommand(void);
GrammarAbs*    Parse_TAG_Lit(void);
GrammarAbs*    Parse_XeqCommand(void);
GrammarAbs*    Parse_XEQ_Lit(void);
GrammarAbs*    Parse_CharCommand(void);
GrammarAbs*    Parse_CHAR_Lit(void);
GrammarAbs*    Parse_NCHAR_Lit(void);
GrammarAbs*    Parse_CharStr(void);
GrammarAbs*    Parse_CharStr2(void);
GrammarAbs*    Parse_CharRangeDef(void);
GrammarAbs*    Parse_CharDef(void);
GrammarAbs*    Parse_EOF_Lit(void);
GrammarAbs*    Parse_AnyCharCommand(void);
GrammarAbs*    Parse_NumStr(void);
GrammarAbs*    Parse_HexStr(void);
GrammarAbs*    Parse_Identifier(void);
GrammarAbs*    Parse_KleinSuffixOpt(void);
GrammarAbs*    Parse_CComment(void);
GrammarAbs*    Parse_CCommentInside(void);
GrammarAbs*    Parse_CppComment(void);
GrammarAbs*    Parse_WSO(void);
GrammarAbs*    Parse_WS(void);
GrammarAbs*    Parse_WS_Element(void);

// XEQ Includes


GrammarAbs*   Parse_BNFC(void)
{
  ///////////////////////////////////////////////////////////
  // GRAM Group
  ///////////////////////////////////////////////////////////

  GRAM  = new GrammarObj;
  GRAM->SetLabel("BNFC");

  ///////////////////////////////////////////////////////////
  // Rule Entry Points
  ///////////////////////////////////////////////////////////

  GRAM->PushBack(Parse_Start());
  GRAM->PushBack(Parse_ParserLabel());
  GRAM->PushBack(Parse_PumpLabel());
  GRAM->PushBack(Parse_Statement());
  GRAM->PushBack(Parse_OrStatement());
  GRAM->PushBack(Parse_OrStatement2());
  GRAM->PushBack(Parse_AndStatement());
  GRAM->PushBack(Parse_AndCommand2());
  GRAM->PushBack(Parse_Command());
  GRAM->PushBack(Parse_CommandType());
  GRAM->PushBack(Parse_OutputListCommandAppendItem());
  GRAM->PushBack(Parse_OutputListCommandAppendChar());
  GRAM->PushBack(Parse_OutputListCommand());
  GRAM->PushBack(Parse_OutputListCommand2());
  GRAM->PushBack(Parse_OutputListStatementLabel());
  GRAM->PushBack(Parse_OutputListStmntRef());
  GRAM->PushBack(Parse_RefCommand());
  GRAM->PushBack(Parse_REF_Lit());
  GRAM->PushBack(Parse_LiteralCommand());
  GRAM->PushBack(Parse_LIT_Lit());
  GRAM->PushBack(Parse_LiteralElement());
  GRAM->PushBack(Parse_LiteralEscapeChar());
  GRAM->PushBack(Parse_HexChar());
  GRAM->PushBack(Parse_TagCommand());
  GRAM->PushBack(Parse_TAG_Lit());
  GRAM->PushBack(Parse_XeqCommand());
  GRAM->PushBack(Parse_XEQ_Lit());
  GRAM->PushBack(Parse_CharCommand());
  GRAM->PushBack(Parse_CHAR_Lit());
  GRAM->PushBack(Parse_NCHAR_Lit());
  GRAM->PushBack(Parse_CharStr());
  GRAM->PushBack(Parse_CharStr2());
  GRAM->PushBack(Parse_CharRangeDef());
  GRAM->PushBack(Parse_CharDef());
  GRAM->PushBack(Parse_EOF_Lit());
  GRAM->PushBack(Parse_AnyCharCommand());
  GRAM->PushBack(Parse_NumStr());
  GRAM->PushBack(Parse_HexStr());
  GRAM->PushBack(Parse_Identifier());
  GRAM->PushBack(Parse_KleinSuffixOpt());
  GRAM->PushBack(Parse_CComment());
  GRAM->PushBack(Parse_CCommentInside());
  GRAM->PushBack(Parse_CppComment());
  GRAM->PushBack(Parse_WSO());
  GRAM->PushBack(Parse_WS());
  GRAM->PushBack(Parse_WS_Element());

  return  GRAM;
}


GrammarAbs*    Parse_Start(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Start'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Start");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'REF(ParserLabel)'
  REF = new RefObj;
  REF->SetLink("ParserLabel");
  AND->PushBack(REF);

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #5: '$(PARSER,4)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("PARSER");
  OUTL->PushRuleNumBack(4);

  // RULE #6: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #7: 'CHAR({)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '{' );

  // RULE #8: 'REF(Statement)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("Statement");
  KLEIN->SetKleinChild(REF);

  // RULE #9: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #10: 'CHAR(})'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '}' );

  // RULE #11: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #12: 'CHAR(EOF)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 256 );

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'REF(PumpLabel)'
  REF = new RefObj;
  REF->SetLink("PumpLabel");
  AND->PushBack(REF);

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #5: '$(PUMP,4)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("PUMP");
  OUTL->PushRuleNumBack(4);

  // RULE #6: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #7: 'CHAR({)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '{' );

  // RULE #8: 'REF(Statement)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("Statement");
  KLEIN->SetKleinChild(REF);

  // RULE #9: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #10: 'CHAR(})'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '}' );

  // RULE #11: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #12: 'CHAR(EOF)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 256 );

  return    OR;
}


GrammarAbs*    Parse_ParserLabel(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'ParserLabel'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("ParserLabel");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(P,p)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'P' );
  CHAR->SetChar( 'p' );

  // RULE #2: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #3: 'CHAR(R,r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'R' );
  CHAR->SetChar( 'r' );

  // RULE #4: 'CHAR(S,s)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'S' );
  CHAR->SetChar( 's' );

  // RULE #5: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #6: 'CHAR(R,r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'R' );
  CHAR->SetChar( 'r' );

  return    OR;
}


GrammarAbs*    Parse_PumpLabel(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'PumpLabel'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("PumpLabel");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(P,p)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'P' );
  CHAR->SetChar( 'p' );

  // RULE #2: 'CHAR(U,u)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'U' );
  CHAR->SetChar( 'u' );

  // RULE #3: 'CHAR(M,m)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'M' );
  CHAR->SetChar( 'm' );

  // RULE #4: 'CHAR(P,p)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'P' );
  CHAR->SetChar( 'p' );

  return    OR;
}


GrammarAbs*    Parse_Statement(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Statement'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Statement");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #3: '$(STMNT,2)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("STMNT");
  OUTL->PushRuleNumBack(2);

  // RULE #4: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #5: 'CHAR(:)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ':' );

  // RULE #6: 'REF(OrStatement)'
  REF = new RefObj;
  REF->SetLink("OrStatement");
  AND->PushBack(REF);

  // RULE #7: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #8: 'CHAR(;)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ';' );

  return    OR;
}


GrammarAbs*    Parse_OrStatement(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'OrStatement'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("OrStatement");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(AndStatement)'
  REF = new RefObj;
  REF->SetLink("AndStatement");
  AND->PushBack(REF);

  // RULE #2: 'REF(OrStatement2)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("OrStatement2");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_OrStatement2(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'OrStatement2'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("OrStatement2");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(|)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '|' );

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'REF(AndStatement)'
  REF = new RefObj;
  REF->SetLink("AndStatement");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_AndStatement(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'AndStatement'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("AndStatement");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: '$(AND)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("AND");

  // RULE #2: 'REF(AndCommand2)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("AndCommand2");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_AndCommand2(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'AndCommand2'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("AndCommand2");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'REF(Command)'
  REF = new RefObj;
  REF->SetLink("Command");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_Command(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Command'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Command");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CommandType)'
  REF = new RefObj;
  REF->SetLink("CommandType");
  AND->PushBack(REF);

  // RULE #2: 'REF(KleinSuffixOpt)'
  REF = new RefObj;
  REF->SetLink("KleinSuffixOpt");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_CommandType(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CommandType'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CommandType");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(RefCommand)'
  REF = new RefObj;
  REF->SetLink("RefCommand");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CharCommand)'
  REF = new RefObj;
  REF->SetLink("CharCommand");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(LiteralCommand)'
  REF = new RefObj;
  REF->SetLink("LiteralCommand");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(OutputListCommandAppendItem)'
  REF = new RefObj;
  REF->SetLink("OutputListCommandAppendItem");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #4
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(OutputListCommandAppendChar)'
  REF = new RefObj;
  REF->SetLink("OutputListCommandAppendChar");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #5
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(OutputListCommand)'
  REF = new RefObj;
  REF->SetLink("OutputListCommand");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #6
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(TagCommand)'
  REF = new RefObj;
  REF->SetLink("TagCommand");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #7
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(XeqCommand)'
  REF = new RefObj;
  REF->SetLink("XeqCommand");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_OutputListCommandAppendItem(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'OutputListCommandAppendItem'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("OutputListCommandAppendItem");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR($)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '$' );

  // RULE #2: 'CHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );

  // RULE #3: 'CHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );

  // RULE #4: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #5: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #6: '$(OUT_LIST_A_CHAR)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("OUT_LIST_A_CHAR");

  // RULE #7: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #8: 'REF(OutputListStmntRef)'
  REF = new RefObj;
  REF->SetLink("OutputListStmntRef");
  AND->PushBack(REF);

  // RULE #9: 'REF(OutputListCommand2)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("OutputListCommand2");
  KLEIN->SetKleinChild(REF);

  // RULE #10: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #11: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  return    OR;
}


GrammarAbs*    Parse_OutputListCommandAppendChar(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'OutputListCommandAppendChar'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("OutputListCommandAppendChar");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR($)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '$' );

  // RULE #2: 'CHAR(>)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '>' );

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #5: '$(OUT_LIST_A_ITEM)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("OUT_LIST_A_ITEM");

  // RULE #6: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #7: 'REF(OutputListStmntRef)'
  REF = new RefObj;
  REF->SetLink("OutputListStmntRef");
  AND->PushBack(REF);

  // RULE #8: 'REF(OutputListCommand2)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("OutputListCommand2");
  KLEIN->SetKleinChild(REF);

  // RULE #9: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #10: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  return    OR;
}


GrammarAbs*    Parse_OutputListCommand(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'OutputListCommand'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("OutputListCommand");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR($)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '$' );

  // RULE #2: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #3: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #4: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #5: 'REF(OutputListStatementLabel)?'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienQuestion);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("OutputListStatementLabel");
  KLEIN->SetKleinChild(REF);

  // RULE #6: '$(OUT_LIST_START,5)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("OUT_LIST_START");
  OUTL->PushRuleNumBack(5);

  // RULE #7: 'REF(OutputListCommand2)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("OutputListCommand2");
  KLEIN->SetKleinChild(REF);

  // RULE #8: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #9: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  return    OR;
}


GrammarAbs*    Parse_OutputListCommand2(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'OutputListCommand2'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("OutputListCommand2");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(,)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ',' );

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: 'REF(OutputListStmntRef)'
  REF = new RefObj;
  REF->SetLink("OutputListStmntRef");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_OutputListStatementLabel(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'OutputListStatementLabel'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("OutputListStatementLabel");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(!-(,*-+,--~)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '!', '(' );
  CHAR->SetRangeChars( '*', '+' );
  CHAR->SetRangeChars( '-', '~' );

  return    OR;
}


GrammarAbs*    Parse_OutputListStmntRef(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'OutputListStmntRef'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("OutputListStmntRef");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(NumStr)'
  REF = new RefObj;
  REF->SetLink("NumStr");
  AND->PushBack(REF);

  // RULE #2: 'LIT()'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("");

  // RULE #3: '$(OUT_LIST_EL,1,2)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("OUT_LIST_EL");
  OUTL->PushRuleNumBack(1);
  OUTL->PushRuleNumBack(2);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(:)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ':' );

  // RULE #3: 'CHAR(:)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ':' );

  // RULE #4: 'REF(NumStr)'
  REF = new RefObj;
  REF->SetLink("NumStr");
  AND->PushBack(REF);

  // RULE #5: '$(OUT_LIST_EL,4,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("OUT_LIST_EL");
  OUTL->PushRuleNumBack(4);
  OUTL->PushRuleNumBack(1);

  return    OR;
}


GrammarAbs*    Parse_RefCommand(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'RefCommand'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("RefCommand");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(REF_Lit)'
  REF = new RefObj;
  REF->SetLink("REF_Lit");
  AND->PushBack(REF);

  // RULE #2: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #3: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #4: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #5: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #6: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #7: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  // RULE #8: '$(REF_CMND,5)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("REF_CMND");
  OUTL->PushRuleNumBack(5);

  return    OR;
}


GrammarAbs*    Parse_REF_Lit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'REF_Lit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("REF_Lit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(R,r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'R' );
  CHAR->SetChar( 'r' );

  // RULE #2: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #3: 'CHAR(F,f)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'F' );
  CHAR->SetChar( 'f' );

  return    OR;
}


GrammarAbs*    Parse_LiteralCommand(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'LiteralCommand'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("LiteralCommand");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(LIT_Lit)'
  REF = new RefObj;
  REF->SetLink("LIT_Lit");
  AND->PushBack(REF);

  // RULE #2: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #3: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #4: '$(LIT_START)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_START");

  // RULE #5: 'REF(LiteralElement)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("LiteralElement");
  KLEIN->SetKleinChild(REF);

  // RULE #6: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  // RULE #7: '$(LIT_END)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_END");

  return    OR;
}


GrammarAbs*    Parse_LIT_Lit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'LIT_Lit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("LIT_Lit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(L,l)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'L' );
  CHAR->SetChar( 'l' );

  // RULE #2: 'CHAR(I,i)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'I' );
  CHAR->SetChar( 'i' );

  // RULE #3: 'CHAR(T,t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'T' );
  CHAR->SetChar( 't' );

  return    OR;
}


GrammarAbs*    Parse_LiteralElement(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'LiteralElement'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("LiteralElement");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\\)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\\' );

  // RULE #2: 'REF(LiteralEscapeChar)'
  REF = new RefObj;
  REF->SetLink("LiteralEscapeChar");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'NCHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );
  CHAR->Invert();

  // RULE #2: '$(LIT_EL,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_EL");
  OUTL->PushRuleNumBack(1);

  return    OR;
}


GrammarAbs*    Parse_LiteralEscapeChar(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'LiteralEscapeChar'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("LiteralEscapeChar");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  // RULE #2: '$(LIT_EL,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_EL");
  OUTL->PushRuleNumBack(1);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\\)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\\' );

  // RULE #2: 'LIT(5)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("5");

  // RULE #3: 'LIT(<)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("<");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(')'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\'' );

  // RULE #2: 'LIT(2)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("2");

  // RULE #3: 'LIT(7)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("7");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(")'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '"' );

  // RULE #2: 'LIT(2)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("2");

  // RULE #3: 'LIT(2)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("2");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #4
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(?)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '?' );

  // RULE #2: 'LIT(3)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("3");

  // RULE #3: 'LIT(?)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("\?");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #5
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'a' );

  // RULE #2: 'LIT(0)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("0");

  // RULE #3: 'LIT(7)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("7");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #6
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(b)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'b' );

  // RULE #2: 'LIT(0)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("0");

  // RULE #3: 'LIT(8)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("8");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #7
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(f)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'f' );

  // RULE #2: 'LIT(0)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("0");

  // RULE #3: 'LIT(<)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("<");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #8
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(n)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'n' );

  // RULE #2: 'LIT(0)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("0");

  // RULE #3: 'LIT(:)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue(":");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #9
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'r' );

  // RULE #2: 'LIT(0)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("0");

  // RULE #3: 'LIT(=)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("=");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #10
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 't' );

  // RULE #2: 'LIT(0)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("0");

  // RULE #3: 'LIT(9)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("9");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #11
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(v)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'v' );

  // RULE #2: 'LIT(0)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("0");

  // RULE #3: 'LIT(;)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue(";");

  // RULE #4: '$(LIT_HEX2,2,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");
  OUTL->PushRuleNumBack(2);
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #12
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(x)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'x' );

  // RULE #2: '$(LIT_HEX2)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("LIT_HEX2");

  // RULE #3: 'REF(HexChar)'
  REF = new RefObj;
  REF->SetLink("HexChar");
  AND->PushBack(REF);

  // RULE #4: 'REF(HexChar)'
  REF = new RefObj;
  REF->SetLink("HexChar");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_HexChar(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'HexChar'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("HexChar");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(0-9)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetRangeChars( '0', '9' );

  // RULE #2: '$>>(1)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(1);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #2: 'LIT(:)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue(":");

  // RULE #3: '$>>(2)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(2);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(B,b)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'B' );
  CHAR->SetChar( 'b' );

  // RULE #2: 'LIT(;)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue(";");

  // RULE #3: '$>>(2)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(2);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(C,c)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'C' );
  CHAR->SetChar( 'c' );

  // RULE #2: 'LIT(<)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("<");

  // RULE #3: '$>>(2)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(2);

  ///////////////////////////////////////////////////////////
  // AND Group #4
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(D,d)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'D' );
  CHAR->SetChar( 'd' );

  // RULE #2: 'LIT(=)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("=");

  // RULE #3: '$>>(2)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(2);

  ///////////////////////////////////////////////////////////
  // AND Group #5
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #2: 'LIT(>)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue(">");

  // RULE #3: '$>>(2)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(2);

  ///////////////////////////////////////////////////////////
  // AND Group #6
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(F,f)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'F' );
  CHAR->SetChar( 'f' );

  // RULE #2: 'LIT(?)'
  LIT = new LiteralObj;
  AND->PushBack(LIT);
  LIT->SetValue("\?");

  // RULE #3: '$>>(2)'
  OUTL = new OutputListObj(OutputListObj::AppendChar);
  AND->PushBack(OUTL);
  OUTL->PushRuleNumBack(2);

  return    OR;
}


GrammarAbs*    Parse_TagCommand(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'TagCommand'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("TagCommand");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(TAG_Lit)'
  REF = new RefObj;
  REF->SetLink("TAG_Lit");
  AND->PushBack(REF);

  // RULE #2: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #3: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #4: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #5: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #6: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #7: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  // RULE #8: '$(TAG,5)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("TAG");
  OUTL->PushRuleNumBack(5);

  return    OR;
}


GrammarAbs*    Parse_TAG_Lit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'TAG_Lit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("TAG_Lit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(T,t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'T' );
  CHAR->SetChar( 't' );

  // RULE #2: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #3: 'CHAR(G,g)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'G' );
  CHAR->SetChar( 'g' );

  return    OR;
}


GrammarAbs*    Parse_XeqCommand(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'XeqCommand'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("XeqCommand");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(XEQ_Lit)'
  REF = new RefObj;
  REF->SetLink("XEQ_Lit");
  AND->PushBack(REF);

  // RULE #2: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #3: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #4: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #5: 'REF(Identifier)'
  REF = new RefObj;
  REF->SetLink("Identifier");
  AND->PushBack(REF);

  // RULE #6: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #7: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  // RULE #8: '$(XEQ,5)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("XEQ");
  OUTL->PushRuleNumBack(5);

  return    OR;
}


GrammarAbs*    Parse_XEQ_Lit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'XEQ_Lit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("XEQ_Lit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(X,x)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'X' );
  CHAR->SetChar( 'x' );

  // RULE #2: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #3: 'CHAR(Q,q)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'Q' );
  CHAR->SetChar( 'q' );

  return    OR;
}


GrammarAbs*    Parse_CharCommand(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CharCommand'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CharCommand");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(NCHAR_Lit)'
  REF = new RefObj;
  REF->SetLink("NCHAR_Lit");
  AND->PushBack(REF);

  // RULE #2: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #3: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #4: '$(CHAR_START)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR_START");

  // RULE #5: 'REF(CharStr)'
  REF = new RefObj;
  REF->SetLink("CharStr");
  AND->PushBack(REF);

  // RULE #6: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  // RULE #7: '$(CHAR_NOT_END)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR_NOT_END");

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CHAR_Lit)'
  REF = new RefObj;
  REF->SetLink("CHAR_Lit");
  AND->PushBack(REF);

  // RULE #2: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #3: 'CHAR(()'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '(' );

  // RULE #4: '$(CHAR_START)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR_START");

  // RULE #5: 'REF(CharStr)'
  REF = new RefObj;
  REF->SetLink("CharStr");
  AND->PushBack(REF);

  // RULE #6: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  return    OR;
}


GrammarAbs*    Parse_CHAR_Lit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CHAR_Lit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CHAR_Lit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(C,c)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'C' );
  CHAR->SetChar( 'c' );

  // RULE #2: 'CHAR(H,h)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'H' );
  CHAR->SetChar( 'h' );

  // RULE #3: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #4: 'CHAR(R,r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'R' );
  CHAR->SetChar( 'r' );

  return    OR;
}


GrammarAbs*    Parse_NCHAR_Lit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'NCHAR_Lit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("NCHAR_Lit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(N,n)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'N' );
  CHAR->SetChar( 'n' );

  // RULE #2: 'CHAR(C,c)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'C' );
  CHAR->SetChar( 'c' );

  // RULE #3: 'CHAR(H,h)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'H' );
  CHAR->SetChar( 'h' );

  // RULE #4: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #5: 'CHAR(R,r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'R' );
  CHAR->SetChar( 'r' );

  return    OR;
}


GrammarAbs*    Parse_CharStr(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CharStr'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CharStr");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #2: 'REF(AnyCharCommand)'
  REF = new RefObj;
  REF->SetLink("AnyCharCommand");
  AND->PushBack(REF);

  // RULE #3: 'REF(WSO)'
  REF = new RefObj;
  REF->SetLink("WSO");
  AND->PushBack(REF);

  // RULE #4: '$(CHAR_ALL)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR_ALL");

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CharRangeDef)'
  REF = new RefObj;
  REF->SetLink("CharRangeDef");
  AND->PushBack(REF);

  // RULE #2: 'REF(CharStr2)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("CharStr2");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_CharStr2(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CharStr2'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CharStr2");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(,)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ',' );

  // RULE #2: 'REF(CharRangeDef)'
  REF = new RefObj;
  REF->SetLink("CharRangeDef");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_CharRangeDef(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CharRangeDef'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CharRangeDef");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CharDef)'
  REF = new RefObj;
  REF->SetLink("CharDef");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(-)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '-' );

  // RULE #3: 'REF(CharDef)'
  REF = new RefObj;
  REF->SetLink("CharDef");
  AND->PushBack(REF);

  // RULE #4: '$(CHAR_RANGE)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR_RANGE");

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CharDef)'
  REF = new RefObj;
  REF->SetLink("CharDef");
  AND->PushBack(REF);

  // RULE #2: '$(CHAR_SINGLE)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR_SINGLE");

  return    OR;
}


GrammarAbs*    Parse_CharDef(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CharDef'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CharDef");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(EOF_Lit)'
  REF = new RefObj;
  REF->SetLink("EOF_Lit");
  AND->PushBack(REF);

  // RULE #2: '$(CHAREOF)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAREOF");

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\\)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\\' );

  // RULE #2: 'CHAR(\\)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\\' );

  // RULE #3: '$(CHAR,2)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR");
  OUTL->PushRuleNumBack(2);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\\)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\\' );

  // RULE #2: 'CHAR(\))'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( ')' );

  // RULE #3: '$(CHAR,2)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR");
  OUTL->PushRuleNumBack(2);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\\)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\\' );

  // RULE #2: 'CHAR(x)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'x' );

  // RULE #3: 'REF(HexStr)'
  REF = new RefObj;
  REF->SetLink("HexStr");
  AND->PushBack(REF);

  // RULE #4: '$(HEXCHAR,3)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("HEXCHAR");
  OUTL->PushRuleNumBack(3);

  ///////////////////////////////////////////////////////////
  // AND Group #4
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(ANY)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->Invert();

  // RULE #2: '$(CHAR,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("CHAR");
  OUTL->PushRuleNumBack(1);

  return    OR;
}


GrammarAbs*    Parse_EOF_Lit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'EOF_Lit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("EOF_Lit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #2: 'CHAR(O,o)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'O' );
  CHAR->SetChar( 'o' );

  // RULE #3: 'CHAR(F,f)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'F' );
  CHAR->SetChar( 'f' );

  return    OR;
}


GrammarAbs*    Parse_AnyCharCommand(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'AnyCharCommand'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("AnyCharCommand");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #2: 'CHAR(N,n)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'N' );
  CHAR->SetChar( 'n' );

  // RULE #3: 'CHAR(Y,y)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'Y' );
  CHAR->SetChar( 'y' );

  return    OR;
}


GrammarAbs*    Parse_NumStr(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'NumStr'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("NumStr");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(0-9)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '0', '9' );

  return    OR;
}


GrammarAbs*    Parse_HexStr(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'HexStr'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("HexStr");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(0-9,A-F,a-f)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '0', '9' );
  CHAR->SetRangeChars( 'A', 'F' );
  CHAR->SetRangeChars( 'a', 'f' );

  return    OR;
}


GrammarAbs*    Parse_Identifier(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Identifier'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Identifier");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(A-Z,a-z)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetRangeChars( 'A', 'Z' );
  CHAR->SetRangeChars( 'a', 'z' );

  // RULE #2: 'CHAR(0-9,A-Z,_,a-z)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '0', '9' );
  CHAR->SetRangeChars( 'A', 'Z' );
  CHAR->SetChar( '_' );
  CHAR->SetRangeChars( 'a', 'z' );

  return    OR;
}


GrammarAbs*    Parse_KleinSuffixOpt(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'KleinSuffixOpt'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("KleinSuffixOpt");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  // RULE #2: '$(KLEIN,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("KLEIN");
  OUTL->PushRuleNumBack(1);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(+)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '+' );

  // RULE #2: '$(KLEIN,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("KLEIN");
  OUTL->PushRuleNumBack(1);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(?)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '?' );

  // RULE #2: '$(KLEIN,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("KLEIN");
  OUTL->PushRuleNumBack(1);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: '$(KLEIN)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("KLEIN");

  return    OR;
}


GrammarAbs*    Parse_CComment(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CComment'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CComment");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #2: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  // RULE #3: 'REF(CCommentInside)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("CCommentInside");
  KLEIN->SetKleinChild(REF);

  // RULE #4: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  // RULE #5: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  return    OR;
}


GrammarAbs*    Parse_CCommentInside(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CCommentInside'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CCommentInside");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  // RULE #2: 'NCHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );
  CHAR->Invert();

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'NCHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );
  CHAR->Invert();

  return    OR;
}


GrammarAbs*    Parse_CppComment(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CppComment'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CppComment");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #2: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #3: 'NCHAR(\x0a)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetChar( '\x0a' );
  CHAR->Invert();

  // RULE #4: 'CHAR(\x0a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\x0a' );

  return    OR;
}


GrammarAbs*    Parse_WSO(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WSO'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WSO");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WS_Element)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("WS_Element");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_WS(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WS'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WS");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WS_Element)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("WS_Element");
  KLEIN->SetKleinChild(REF);

  return    OR;
}


GrammarAbs*    Parse_WS_Element(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WS_Element'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WS_Element");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\x09-\x0a,\x0d,\x20)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '\x09', '\x0a' );
  CHAR->SetChar( '\x0d' );
  CHAR->SetChar( '\x20' );

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CComment)'
  REF = new RefObj;
  REF->SetLink("CComment");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CppComment)'
  REF = new RefObj;
  REF->SetLink("CppComment");
  AND->PushBack(REF);

  return    OR;
}



}
}

