///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  InverseParseFrame.cpp                                                             //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include  <cassert>
#include  <iostream>
#include  <iomanip>
#include  <sstream>
#include  "InverseParseFrame.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
// InverseParseFrame::~InverseParseFrame
///////////////////////////////////////////////////////////////////////////////////////////////////

InverseParseFrame::~InverseParseFrame(void)
{
  // Move file to mark
  if( Out.tellp() != FileP )
  {
    MoveFileTo(FileP);
  }

  // Move list to mark if not there
  if( this->ListP != this->CurrListPos )
  {
    this->CurrListPos   = this->ListP;
  }

  if( this->Father )
  {
    // Copy list position to father as there is no inherent position
    this->Father->CurrListPos   = this->CurrListPos;

    // Copy the 'ListInInit' flag up as well
    this->Father->ListInInit    = this->ListInInit;
  }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// InverseParseFrame::Display
///////////////////////////////////////////////////////////////////////////////////////////////////

string    InverseParseFrame::Display(void) const
{
  ostringstream   oS;
  unsigned int    ListDepth = 0, ListDepth0 = 0;

  // List position 1
  for( auto L = this->List.Begin() ; L != this->ListP0 ; ++L, ++ListDepth0 );
  for( auto L = this->List.Begin() ; L != this->ListP  ; ++L, ++ListDepth  );

  if(this->ListInInit)
  {
    oS << "IPF(List#=<" << ListDepth0 << "," << ListDepth << ">  File=<" << FileP0 << "," << FileP << ">)";
  }
  else
  {
    oS << "IPF(List=<" << ListDepth0 << "," << ListDepth << ">  File=<" << FileP0 << "," << FileP << ">)";
  }

  return  oS.str();
}
}   // End namespace 'wej_parser'


