///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  KleinWrapperObj.h                                                                 //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include    <cassert>
#include    <sstream>
#include    "GrammarAbs.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
// class  KleinWrapperObj : public GrammarAbs
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class   KleinWrapperObj : public GrammarAbs
{
private:

  GrammarAbs*                 Child;
  GrammarAbs::KlienFlagsType  Klein;

public:

  inline KleinWrapperObj(void) : Child(nullptr), Klein(GrammarAbs::NoKlien)
  {
  }

  inline virtual ~KleinWrapperObj()
  {
    if(Child)   delete Child;
  }

  inline void        SetKlein( const GrammarAbs::KlienFlagsType  NewKlein )
  {
    Klein = NewKlein;
  }

  inline GrammarAbs::KlienFlagsType      GetKlein(void) const
  {
    return  Klein;
  }

  inline void           SetKleinChild( GrammarAbs  *Son )
  {
    Child  = Son;
    Son->SetFather(this);
  }

  inline void           ClearKleinChild(void)
  {
    Child = nullptr;
  }

  inline GrammarAbs*    GetKlienChild(void)
  {
    return  Child;
  }

  //////////////////////////////
  // Interfaces
  //////////////////////////////

  string        Display( bool SubRule=false, const unsigned int Inden=0 ) const;
  bool          Parse( ParseFrame PF );
  string        GetOutput( ParseFrame& PF, const unsigned int RuleNum );
  void          GenerateCppParseTree( IBnfToCppCov& OutputData, bool KleinLayer, string& Includes );

  inline bool   DoesRead(void) const
  {
    return  true;
  }

  bool        InverseParse( InverseParseFrame IPF );
};
}   // End namespace 'wej_parser'


