///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  PARSER_Template.cpp                                                               //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include        <cassert>
#include        <iostream>
#include        <iomanip>
#include        <sstream>
#include        <typeinfo>
#include        "GrammarAbs.h"
#include        "GrammarObj.h"
#include        "CharacterObj.h"
#include        "BNFC_Gen.h"
#include        "KleinWrapperObj.h"
#include        "RefObj.h"
#include        "LiteralObj.h"
#include        "OutputListObj.h"
#include        "AndRuleObj.h"
#include        "OrRuleObj.h"
#include        "PARSER_Template.h"

using namespace std;
using namespace wej_parser;


namespace wej_parser
{
namespace wej_template
{
// Variables used in subroutines
GrammarObj*       GRAM;
CharacterObj*     CHAR;
KleinWrapperObj*  KLEIN;
RefObj*           REF;
LiteralObj*       LIT;
OutputListObj*    OUTL;
AndRuleObj*       AND;
OrRuleObj*        OR;

// Function Prototypes
GrammarAbs*    Parse_Start(void);
GrammarAbs*    Parse_Command(void);
GrammarAbs*    Parse_CommandPrefix(void);
GrammarAbs*    Parse_CommandPostfix(void);
GrammarAbs*    Parse_DeclareLiteral(void);
GrammarAbs*    Parse_ValLit(void);
GrammarAbs*    Parse_BarValLit(void);
GrammarAbs*    Parse_TagLit(void);
GrammarAbs*    Parse_PathVal(void);
GrammarAbs*    Parse_IterVal(void);
GrammarAbs*    Parse_EndIter(void);
GrammarAbs*    Parse_BTW_Lit(void);
GrammarAbs*    Parse_XML_Path(void);
GrammarAbs*    Parse_Seg(void);
GrammarAbs*    Parse_Seg_Last(void);
GrammarAbs*    Parse_Label(void);
GrammarAbs*    Parse_String(void);
GrammarAbs*    Parse_StringInner(void);
GrammarAbs*    Parse_TwoBraces_Left(void);
GrammarAbs*    Parse_ThreeBraces_Left(void);
GrammarAbs*    Parse_FourBraces_Left(void);
GrammarAbs*    Parse_FiveBraces_Left(void);
GrammarAbs*    Parse_TwoBraces_Right(void);
GrammarAbs*    Parse_ThreeBraces_Right(void);
GrammarAbs*    Parse_FourBraces_Right(void);
GrammarAbs*    Parse_FiveBraces_Right(void);
GrammarAbs*    Parse_WhitespaceSeqOpt_DontEatLF(void);
GrammarAbs*    Parse_WhitespaceSeqOpt(void);
GrammarAbs*    Parse_WhitespaceSeq(void);
GrammarAbs*    Parse_Whitespace(void);

// XEQ Includes


GrammarAbs*   Parse_Path(void)
{
  ///////////////////////////////////////////////////////////
  // GRAM Group
  ///////////////////////////////////////////////////////////

  GRAM  = new GrammarObj;
  GRAM->SetLabel("Path");

  ///////////////////////////////////////////////////////////
  // Rule Entry Points
  ///////////////////////////////////////////////////////////

  GRAM->PushBack(Parse_Start());
  GRAM->PushBack(Parse_Command());
  GRAM->PushBack(Parse_CommandPrefix());
  GRAM->PushBack(Parse_CommandPostfix());
  GRAM->PushBack(Parse_DeclareLiteral());
  GRAM->PushBack(Parse_ValLit());
  GRAM->PushBack(Parse_BarValLit());
  GRAM->PushBack(Parse_TagLit());
  GRAM->PushBack(Parse_PathVal());
  GRAM->PushBack(Parse_IterVal());
  GRAM->PushBack(Parse_EndIter());
  GRAM->PushBack(Parse_BTW_Lit());
  GRAM->PushBack(Parse_XML_Path());
  GRAM->PushBack(Parse_Seg());
  GRAM->PushBack(Parse_Seg_Last());
  GRAM->PushBack(Parse_Label());
  GRAM->PushBack(Parse_String());
  GRAM->PushBack(Parse_StringInner());
  GRAM->PushBack(Parse_TwoBraces_Left());
  GRAM->PushBack(Parse_ThreeBraces_Left());
  GRAM->PushBack(Parse_FourBraces_Left());
  GRAM->PushBack(Parse_FiveBraces_Left());
  GRAM->PushBack(Parse_TwoBraces_Right());
  GRAM->PushBack(Parse_ThreeBraces_Right());
  GRAM->PushBack(Parse_FourBraces_Right());
  GRAM->PushBack(Parse_FiveBraces_Right());
  GRAM->PushBack(Parse_WhitespaceSeqOpt_DontEatLF());
  GRAM->PushBack(Parse_WhitespaceSeqOpt());
  GRAM->PushBack(Parse_WhitespaceSeq());
  GRAM->PushBack(Parse_Whitespace());

  return  GRAM;
}


GrammarAbs*    Parse_Start(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Start'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Start");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(Command)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("Command");
  KLEIN->SetKleinChild(REF);

  // RULE #2: 'CHAR(EOF)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 256 );

  return    OR;
}


GrammarAbs*    Parse_Command(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Command'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Command");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CommandPrefix)'
  REF = new RefObj;
  REF->SetLink("CommandPrefix");
  AND->PushBack(REF);

  // RULE #2: 'REF(BTW_Lit)'
  REF = new RefObj;
  REF->SetLink("BTW_Lit");
  AND->PushBack(REF);

  // RULE #3: 'REF(WhitespaceSeq)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeq");
  AND->PushBack(REF);

  // RULE #4: 'REF(String)'
  REF = new RefObj;
  REF->SetLink("String");
  AND->PushBack(REF);

  // RULE #5: 'REF(CommandPostfix)'
  REF = new RefObj;
  REF->SetLink("CommandPostfix");
  AND->PushBack(REF);

  // RULE #6: '$(BTW,4)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("BTW");
  OUTL->PushRuleNumBack(4);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CommandPrefix)'
  REF = new RefObj;
  REF->SetLink("CommandPrefix");
  AND->PushBack(REF);

  // RULE #2: 'REF(ValLit)'
  REF = new RefObj;
  REF->SetLink("ValLit");
  AND->PushBack(REF);

  // RULE #3: 'REF(WhitespaceSeq)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeq");
  AND->PushBack(REF);

  // RULE #4: 'REF(XML_Path)'
  REF = new RefObj;
  REF->SetLink("XML_Path");
  AND->PushBack(REF);

  // RULE #5: 'REF(CommandPostfix)'
  REF = new RefObj;
  REF->SetLink("CommandPostfix");
  AND->PushBack(REF);

  // RULE #6: '$(VAL,4)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("VAL");
  OUTL->PushRuleNumBack(4);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CommandPrefix)'
  REF = new RefObj;
  REF->SetLink("CommandPrefix");
  AND->PushBack(REF);

  // RULE #2: 'REF(BarValLit)'
  REF = new RefObj;
  REF->SetLink("BarValLit");
  AND->PushBack(REF);

  // RULE #3: 'REF(WhitespaceSeq)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeq");
  AND->PushBack(REF);

  // RULE #4: 'REF(XML_Path)'
  REF = new RefObj;
  REF->SetLink("XML_Path");
  AND->PushBack(REF);

  // RULE #5: 'REF(CommandPostfix)'
  REF = new RefObj;
  REF->SetLink("CommandPostfix");
  AND->PushBack(REF);

  // RULE #6: '$(BARVAL,4)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("BARVAL");
  OUTL->PushRuleNumBack(4);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CommandPrefix)'
  REF = new RefObj;
  REF->SetLink("CommandPrefix");
  AND->PushBack(REF);

  // RULE #2: 'REF(IterVal)'
  REF = new RefObj;
  REF->SetLink("IterVal");
  AND->PushBack(REF);

  // RULE #3: 'REF(WhitespaceSeq)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeq");
  AND->PushBack(REF);

  // RULE #4: 'REF(XML_Path)'
  REF = new RefObj;
  REF->SetLink("XML_Path");
  AND->PushBack(REF);

  // RULE #5: 'REF(CommandPostfix)'
  REF = new RefObj;
  REF->SetLink("CommandPostfix");
  AND->PushBack(REF);

  // RULE #6: '$(ITER,4)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("ITER");
  OUTL->PushRuleNumBack(4);

  ///////////////////////////////////////////////////////////
  // AND Group #4
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(CommandPrefix)'
  REF = new RefObj;
  REF->SetLink("CommandPrefix");
  AND->PushBack(REF);

  // RULE #2: 'REF(EndIter)'
  REF = new RefObj;
  REF->SetLink("EndIter");
  AND->PushBack(REF);

  // RULE #3: 'REF(CommandPostfix)'
  REF = new RefObj;
  REF->SetLink("CommandPostfix");
  AND->PushBack(REF);

  // RULE #4: '$(END_ITER)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("END_ITER");

  ///////////////////////////////////////////////////////////
  // AND Group #5
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(ANY)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->Invert();

  // RULE #2: '$(ORIG,1)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("ORIG");
  OUTL->PushRuleNumBack(1);

  return    OR;
}


GrammarAbs*    Parse_CommandPrefix(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CommandPrefix'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CommandPrefix");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(FiveBraces_Left)'
  REF = new RefObj;
  REF->SetLink("FiveBraces_Left");
  AND->PushBack(REF);

  // RULE #2: 'CHAR({)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '{' );

  // RULE #3: 'CHAR(5)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '5' );

  // RULE #4: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(FourBraces_Left)'
  REF = new RefObj;
  REF->SetLink("FourBraces_Left");
  AND->PushBack(REF);

  // RULE #2: 'CHAR({)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '{' );

  // RULE #3: 'CHAR(4)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '4' );

  // RULE #4: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(ThreeBraces_Left)'
  REF = new RefObj;
  REF->SetLink("ThreeBraces_Left");
  AND->PushBack(REF);

  // RULE #2: 'CHAR({)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '{' );

  // RULE #3: 'CHAR(3)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '3' );

  // RULE #4: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(TwoBraces_Left)'
  REF = new RefObj;
  REF->SetLink("TwoBraces_Left");
  AND->PushBack(REF);

  // RULE #2: 'CHAR({)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '{' );

  // RULE #3: 'CHAR(2)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '2' );

  // RULE #4: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #4
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR({)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '{' );

  // RULE #2: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_CommandPostfix(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'CommandPostfix'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("CommandPostfix");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(5)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '5' );

  // RULE #3: 'CHAR(})'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '}' );

  // RULE #4: 'REF(FiveBraces_Right)'
  REF = new RefObj;
  REF->SetLink("FiveBraces_Right");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(4)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '4' );

  // RULE #3: 'CHAR(})'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '}' );

  // RULE #4: 'REF(FourBraces_Right)'
  REF = new RefObj;
  REF->SetLink("FourBraces_Right");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(3)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '3' );

  // RULE #3: 'CHAR(})'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '}' );

  // RULE #4: 'REF(ThreeBraces_Right)'
  REF = new RefObj;
  REF->SetLink("ThreeBraces_Right");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #3
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(2)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '2' );

  // RULE #3: 'CHAR(})'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '}' );

  // RULE #4: 'REF(TwoBraces_Right)'
  REF = new RefObj;
  REF->SetLink("TwoBraces_Right");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #4
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(})'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '}' );

  return    OR;
}


GrammarAbs*    Parse_DeclareLiteral(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'DeclareLiteral'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("DeclareLiteral");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(D,d)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'D' );
  CHAR->SetChar( 'd' );

  // RULE #3: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #4: 'CHAR(C,c)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'C' );
  CHAR->SetChar( 'c' );

  // RULE #5: 'CHAR(L,l)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'L' );
  CHAR->SetChar( 'l' );

  // RULE #6: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #7: 'CHAR(R,r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'R' );
  CHAR->SetChar( 'r' );

  // RULE #8: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  return    OR;
}


GrammarAbs*    Parse_ValLit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'ValLit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("ValLit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(V,v)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'V' );
  CHAR->SetChar( 'v' );

  // RULE #3: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #4: 'CHAR(L,l)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'L' );
  CHAR->SetChar( 'l' );

  return    OR;
}


GrammarAbs*    Parse_BarValLit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'BarValLit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("BarValLit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #3: 'CHAR(V,v)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'V' );
  CHAR->SetChar( 'v' );

  // RULE #4: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #5: 'CHAR(L,l)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'L' );
  CHAR->SetChar( 'l' );

  return    OR;
}


GrammarAbs*    Parse_TagLit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'TagLit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("TagLit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(T,t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'T' );
  CHAR->SetChar( 't' );

  // RULE #3: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #4: 'CHAR(G,g)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'G' );
  CHAR->SetChar( 'g' );

  return    OR;
}


GrammarAbs*    Parse_PathVal(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'PathVal'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("PathVal");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(P,p)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'P' );
  CHAR->SetChar( 'p' );

  // RULE #3: 'CHAR(A,a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'A' );
  CHAR->SetChar( 'a' );

  // RULE #4: 'CHAR(T,t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'T' );
  CHAR->SetChar( 't' );

  // RULE #5: 'CHAR(H,h)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'H' );
  CHAR->SetChar( 'h' );

  return    OR;
}


GrammarAbs*    Parse_IterVal(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'IterVal'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("IterVal");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(I,i)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'I' );
  CHAR->SetChar( 'i' );

  // RULE #3: 'CHAR(T,t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'T' );
  CHAR->SetChar( 't' );

  // RULE #4: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #5: 'CHAR(R,r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'R' );
  CHAR->SetChar( 'r' );

  return    OR;
}


GrammarAbs*    Parse_EndIter(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'EndIter'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("EndIter");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #3: 'CHAR(N,n)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'N' );
  CHAR->SetChar( 'n' );

  // RULE #4: 'CHAR(D,d)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'D' );
  CHAR->SetChar( 'd' );

  // RULE #5: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #6: 'CHAR(I,i)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'I' );
  CHAR->SetChar( 'i' );

  // RULE #7: 'CHAR(T,t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'T' );
  CHAR->SetChar( 't' );

  // RULE #8: 'CHAR(E,e)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'E' );
  CHAR->SetChar( 'e' );

  // RULE #9: 'CHAR(R,r)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'R' );
  CHAR->SetChar( 'r' );

  return    OR;
}


GrammarAbs*    Parse_BTW_Lit(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'BTW_Lit'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("BTW_Lit");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(B,b)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'B' );
  CHAR->SetChar( 'b' );

  // RULE #3: 'CHAR(T,t)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'T' );
  CHAR->SetChar( 't' );

  // RULE #4: 'CHAR(W,w)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( 'W' );
  CHAR->SetChar( 'w' );

  return    OR;
}


GrammarAbs*    Parse_XML_Path(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'XML_Path'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("XML_Path");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #3: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #4: 'REF(Seg)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("Seg");
  KLEIN->SetKleinChild(REF);

  // RULE #5: 'REF(Seg_Last)'
  REF = new RefObj;
  REF->SetLink("Seg_Last");
  AND->PushBack(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  // RULE #3: 'REF(Seg)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("Seg");
  KLEIN->SetKleinChild(REF);

  // RULE #4: 'REF(Seg_Last)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("Seg_Last");
  KLEIN->SetKleinChild(REF);

  ///////////////////////////////////////////////////////////
  // AND Group #2
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'REF(Seg)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("Seg");
  KLEIN->SetKleinChild(REF);

  // RULE #3: 'REF(Seg_Last)'
  REF = new RefObj;
  REF->SetLink("Seg_Last");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_Seg(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Seg'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Seg");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'REF(Label)'
  REF = new RefObj;
  REF->SetLink("Label");
  AND->PushBack(REF);

  // RULE #3: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #4: 'CHAR(/)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '/' );

  return    OR;
}


GrammarAbs*    Parse_Seg_Last(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Seg_Last'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Seg_Last");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #2: 'REF(Label)'
  REF = new RefObj;
  REF->SetLink("Label");
  AND->PushBack(REF);

  // RULE #3: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  // RULE #4: 'CHAR(/)?'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienQuestion);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetChar( '/' );

  return    OR;
}


GrammarAbs*    Parse_Label(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Label'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Label");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(A-Z,_,a-z)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetRangeChars( 'A', 'Z' );
  CHAR->SetChar( '_' );
  CHAR->SetRangeChars( 'a', 'z' );

  // RULE #2: 'CHAR(--.,0-9,A-Z,_,a-z)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '-', '.' );
  CHAR->SetRangeChars( '0', '9' );
  CHAR->SetRangeChars( 'A', 'Z' );
  CHAR->SetChar( '_' );
  CHAR->SetRangeChars( 'a', 'z' );

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(.)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '.' );

  // RULE #2: 'CHAR(.)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '.' );

  return    OR;
}


GrammarAbs*    Parse_String(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'String'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("String");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(")'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '"' );

  // RULE #2: 'REF(StringInner)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  REF = new RefObj;
  REF->SetLink("StringInner");
  KLEIN->SetKleinChild(REF);

  // RULE #3: 'CHAR(")'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '"' );

  // RULE #4: '$(STR,2)'
  OUTL = new OutputListObj(OutputListObj::NewEntry);
  AND->PushBack(OUTL);
  OUTL->SetLabel("STR");
  OUTL->PushRuleNumBack(2);

  return    OR;
}


GrammarAbs*    Parse_StringInner(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'StringInner'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("StringInner");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'NCHAR(")'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '"' );
  CHAR->Invert();

  ///////////////////////////////////////////////////////////
  // AND Group #1
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\\)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\\' );

  // RULE #2: 'CHAR(*)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '*' );

  return    OR;
}


GrammarAbs*    Parse_TwoBraces_Left(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'TwoBraces_Left'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("TwoBraces_Left");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt_DontEatLF)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt_DontEatLF");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_ThreeBraces_Left(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'ThreeBraces_Left'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("ThreeBraces_Left");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\x0a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\x0a' );

  // RULE #2: 'REF(WhitespaceSeqOpt_DontEatLF)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt_DontEatLF");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_FourBraces_Left(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'FourBraces_Left'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("FourBraces_Left");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt_DontEatLF)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt_DontEatLF");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(\x0a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\x0a' );

  // RULE #3: 'REF(WhitespaceSeqOpt_DontEatLF)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt_DontEatLF");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_FiveBraces_Left(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'FiveBraces_Left'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("FiveBraces_Left");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_TwoBraces_Right(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'TwoBraces_Right'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("TwoBraces_Right");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt_DontEatLF)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt_DontEatLF");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_ThreeBraces_Right(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'ThreeBraces_Right'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("ThreeBraces_Right");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt_DontEatLF)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt_DontEatLF");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(\x0a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\x0a' );

  return    OR;
}


GrammarAbs*    Parse_FourBraces_Right(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'FourBraces_Right'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("FourBraces_Right");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt_DontEatLF)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt_DontEatLF");
  AND->PushBack(REF);

  // RULE #2: 'CHAR(\x0a)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetChar( '\x0a' );

  // RULE #3: 'REF(WhitespaceSeqOpt_DontEatLF)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt_DontEatLF");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_FiveBraces_Right(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'FiveBraces_Right'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("FiveBraces_Right");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'REF(WhitespaceSeqOpt)'
  REF = new RefObj;
  REF->SetLink("WhitespaceSeqOpt");
  AND->PushBack(REF);

  return    OR;
}


GrammarAbs*    Parse_WhitespaceSeqOpt_DontEatLF(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WhitespaceSeqOpt_DontEatLF'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WhitespaceSeqOpt_DontEatLF");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\x09,\x0d,\x20)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetChar( '\x09' );
  CHAR->SetChar( '\x0d' );
  CHAR->SetChar( '\x20' );

  return    OR;
}


GrammarAbs*    Parse_WhitespaceSeqOpt(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WhitespaceSeqOpt'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WhitespaceSeqOpt");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\x09-\x0a,\x0d,\x20)*'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienStar);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '\x09', '\x0a' );
  CHAR->SetChar( '\x0d' );
  CHAR->SetChar( '\x20' );

  return    OR;
}


GrammarAbs*    Parse_WhitespaceSeq(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'WhitespaceSeq'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("WhitespaceSeq");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\x09-\x0a,\x0d,\x20)+'
  KLEIN    = new KleinWrapperObj;
  KLEIN->SetKlein(GrammarAbs::KlienPlus);
  AND->PushBack(KLEIN);
  CHAR = new CharacterObj;
  KLEIN->SetKleinChild(CHAR);
  CHAR->SetRangeChars( '\x09', '\x0a' );
  CHAR->SetChar( '\x0d' );
  CHAR->SetChar( '\x20' );

  return    OR;
}


GrammarAbs*    Parse_Whitespace(void)
{
  ///////////////////////////////////////////////////////////
  // OR Group 'Whitespace'
  ///////////////////////////////////////////////////////////

  OR  = new OrRuleObj;
  OR->SetLabel("Whitespace");

  ///////////////////////////////////////////////////////////
  // AND Group #0
  ///////////////////////////////////////////////////////////

  AND = new AndRuleObj;
  OR->PushBack(AND);

  // RULE #1: 'CHAR(\x09-\x0a,\x0d,\x20)'
  CHAR = new CharacterObj;
  AND->PushBack(CHAR);
  CHAR->SetRangeChars( '\x09', '\x0a' );
  CHAR->SetChar( '\x0d' );
  CHAR->SetChar( '\x20' );

  return    OR;
}



}
}

