///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  ParseFrame.cpp                                                                    //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#include    <cassert>
#include    <iostream>
#include    <iomanip>
#include    <sstream>
#include    <typeinfo>
#include    "ParseFrame.h"
#include    "AndRuleObj.h"
#include    "OrRuleObj.h"
#include    "GrammarObj.h"

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::~ParseFrame
///////////////////////////////////////////////////////////////////////////////////////////////////

ParseFrame::~ParseFrame(void)
{
  // Move file to mark
  if( IN.tellg() != PosIn )
  {
    this->MoveInFileTo(PosIn);
  }

  // Restore list
  this->MoveOutListTo( PosLOut, PosL2Out, PosL2SOut );
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// void   ParseFrame::OutputMatchedList
///////////////////////////////////////////////////////////////////////////////////////////////////

void  ParseFrame::OutputMatchedList(void)
{
  // Get current positions
  streampos   dIn = PosIn - PosIn0;

  assert( dIn >= 0 );

  // No data to move
  if( dIn == 0 )    return;

  // Make a buffer to hold the data to transfer
  char    *Buf = new char[dIn];

  // Re-position the IN channel to get the data
  MoveInFileTo(PosIn0);

  // Copy data
  IN.read( Buf, dIn );

  assert( IN.gcount() == dIn );

  // ListOut:  Push a new string onto the end vector
  assert( ListOut.GetDataList().size() > 0 );

  ListOut.GetDataList().push_back(OutputRec());
  ListOut.GetDataList().back().GetDataVect().push_back( string( Buf, IN.gcount() ) );

  // Delete buffer
  delete[]    Buf;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// unsigned int   ParseFrame::ReadFromInput
///////////////////////////////////////////////////////////////////////////////////////////////////

// Returns the number of characters actual read
unsigned int   ParseFrame::ReadFromInput( const unsigned int Length, string& Result ) const
{
  if( Length == 0 )   return  0;

  // Allocate a temp buffer
  char   *Buf = new char[Length];

  // Read the data
  IN.read( Buf, Length );

  assert( IN.gcount() == (int) Length );

  // Read the data
  Result    = string(Buf,Length);

  // Delete the buffer
  delete[]    Buf;

  // Return the number of characters
  return    IN.gcount();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// string    ParseFrame::Display
///////////////////////////////////////////////////////////////////////////////////////////////////

string    ParseFrame::Display( const Outputs& OutList )
{
  list<OutputRec>::const_iterator   I;
  ostringstream                     oS;

  for( I = OutList.GetDataList().begin() ; I != OutList.GetDataList().end() ; ++I )
  {
    oS << ParseFrame::Display(*I);
  }

  return    oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// string    ParseFrame::Display
///////////////////////////////////////////////////////////////////////////////////////////////////

string    ParseFrame::Display( const OutputRec&  OutRec )
{
  ostringstream   oS;

  for( unsigned int K = 0 ; K < OutRec.GetDataVect().size() ; ++K )
  {
    if( K > 0 )     oS << ", ";
    oS << OutRec.GetDataVect()[K];
  }

  oS << endl;

  return    oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::Display
///////////////////////////////////////////////////////////////////////////////////////////////////

string    ParseFrame::Display(void) const
{
  ostringstream   oS;

  oS << "PF(List=<" << PosLOut0 << "," << PosLOut << "> File=<" << PosIn0 << "," << PosIn << ">)";

  return  oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::Display2
///////////////////////////////////////////////////////////////////////////////////////////////////

string    ParseFrame::Display2(void) const
{
  ostringstream   oS;

  oS << "PF[" << this << "] (List=<" << PosLOut0 << "," << PosLOut << "> File=<" << PosIn0
     << "," << PosIn << ">)";
  oS << "   father=" << this->Father << "  rule=" << this->Rule << endl;

  return  oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::Display3
///////////////////////////////////////////////////////////////////////////////////////////////////

string    ParseFrame::Display3(void) const
{
  ostringstream   oS;

  for( auto K=this ; K ; K = K->Father )
  {
    oS << K->Display2();
  }

  return  oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::SetRule( GrammarAbs* pRule )
///////////////////////////////////////////////////////////////////////////////////////////////////

void  ParseFrame::SetRule( GrammarAbs* pRule )
{
  // save pointer to the rule
  this->Rule  = pRule;

  // See if 'pRule' is an 'AndRuleObj'
  if( typeid(*pRule) != typeid(AndRuleObj) )  return;

  // Cast a specific pointer to this 'AndRuleObj' structure
  AndRuleObj*     pAnd = dynamic_cast<AndRuleObj*>(pRule);

  assert(pAnd);

  // Allocate space for the input stream start positions so we can recover output
  this->StartInputCharPos.resize( pAnd->GetNumberOfRules() + 1 );
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::GetSegmentChars( const unsigned int SegmentNum )
///////////////////////////////////////////////////////////////////////////////////////////////////

string    ParseFrame::GetSegmentChars( const unsigned int SegmentNum )
{
  string    R;

  // Make sure there is something to move
  if( (SegmentNum) > this->StartInputCharPos.size() )   return  R;

  // Get the start and stop character numbers
  long int    StartChar = (long int) this->StartInputCharPos[SegmentNum-1];
  long int    EndChar   = (long int) this->StartInputCharPos[SegmentNum  ] - 1;
  long int    NumChars  = EndChar - StartChar + 1;
  streampos   InCurr    = IN.tellg();

  // See if there is anything to do
  if( NumChars <= 0 )   return  R;

  // Move to input start of sequence
  this->MoveInFileTo( StartChar );

  // Read the data at this segment
  this->ReadFromInput( NumChars, R );

  // Move IN stream back to the original position
  this->MoveInFileTo( InCurr );

  return    R;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::SaveInputFilePosition( const unsigned int StatementRuleNum )
///////////////////////////////////////////////////////////////////////////////////////////////////

void    ParseFrame::SaveInputFilePosition( const unsigned int StatementRuleNum )
{
  assert( typeid(*this->GetRule()) == typeid(AndRuleObj) );

  this->StartInputCharPos[StatementRuleNum]   = this->GetInPos();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::FindUpperAndRule( const string RuleName )
///////////////////////////////////////////////////////////////////////////////////////////////////
// If 'RuleName' is not specified then the next AndRuleObj up is returned.  Returns nullptr
//     if not found.  'PF_AtAnd' is the ParseFrame at the 'AndRuleObj'.
AndRuleObj*    ParseFrame::FindUpperAndRule(  ParseFrame** PF_AtAnd, const string RuleName )
{
  auto            P     = this->Father;
  AndRuleObj*     ARet  = nullptr;

  // Search upward for the next AndRuleObj with the right name
  for(  ; P ; P = P->Father )
  {
    // Search upward for the next AndRuleObj
    for(  ; P ; P = P->Father )
    {
      // Must be an 'AndRuleObj' type
      if( typeid(*(P->GetRule())) == typeid(AndRuleObj) )   break;
    }

    // Didn't find it
    if( !P )  return  nullptr;

    // Cast a pointer to this AndRuleObj
    ARet  = dynamic_cast<AndRuleObj*>(P->GetRule());

    // If no rule name is specified then we're done
    if( RuleName.empty() )
    {
      *PF_AtAnd   = P;
      return    ARet;
    }

    // Now search upward for the OrRuleObj
    for(  ; P ; P = P->Father )
    {
      // Must be an 'AndRuleObj' type
      if( typeid(*(P->GetRule())) == typeid(OrRuleObj) )   break;
    }

    // Didn't find it
    if( !P )  return  nullptr;

    //  Make sure the rule name matches
    if( P->GetRule()->GetLabel() == RuleName )   break;
  }

  // See if we found it
  if( !P )  return  nullptr;

  // We found it
  *PF_AtAnd   = P;

  return    ARet;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::GetRuleDebugDescriptor(void) const
///////////////////////////////////////////////////////////////////////////////////////////////////

string    ParseFrame::GetRuleDebugDescriptor(void) const
{
  ostringstream   oS;
  bool            First = true;
  GrammarAbs*     LastRule = nullptr;
  unsigned int    LastAndRuleNum = 0;

  // PF->Rule = nullptr marks tp of grammar
  for( const ParseFrame*  PF = this ; PF ; LastRule = PF->Rule, PF = PF->Father )
  {
    // Handle an AND rule
    if( typeid(*PF->Rule) == typeid(AndRuleObj) )
    {
      AndRuleObj*   AndPtr = dynamic_cast<AndRuleObj*>(PF->Rule);

      // Find the rule number
      for( auto K=1U ; K <= AndPtr->GetNumberOfRules() ; ++K )
      {
        if( LastRule != AndPtr->GetRule(K) )  continue;

        // Got a match
        LastAndRuleNum = K;

        break;
      }
    }

    // Handle an OR rule
    else if( typeid(*PF->Rule) == typeid(OrRuleObj) )
    {
      if( !First )   oS << '.';
      First = false;
      oS << PF->Rule->GetLabel();

      OrRuleObj*     OrPtr = dynamic_cast<OrRuleObj*>(PF->Rule);

      // Find the rule number
      for( auto K=1U ; K <= OrPtr->GetNumberOfRules() ; ++K )
      {
        if( LastRule != OrPtr->GetRule(K) )   continue;

        // Got a match
        if( LastAndRuleNum != 0 )
        {
          oS << "<" << K << "," << LastAndRuleNum << ">";
        }
        else
        {
          oS << "<" << K << ">";
        }

        break;
      }
    }

    // If we're at the top return
    if( PF->IsTopmost() )   break;
  }

  return  oS.str();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::IsTopmost(void) const
///////////////////////////////////////////////////////////////////////////////////////////////////

// The PF corresponding to the GrammarObj
bool    ParseFrame::IsTopmost(void) const
{
  return    (this->Father==nullptr) ||
            (this->Rule==nullptr) ||
            (typeid(*this->Rule)==typeid(GrammarObj));
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::GetOutList2Pos(void) const
///////////////////////////////////////////////////////////////////////////////////////////////////

vector<string>::size_type   ParseFrame::GetOutList2Pos(void) const
{
  if( ListOut.GetDataList().size() == 0 )   return    0;
  return    ListOut.GetDataList().back().Data.size();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::GetOutList2SPos(void) const
///////////////////////////////////////////////////////////////////////////////////////////////////

string::size_type   ParseFrame::GetOutList2SPos(void) const
{
  if( ListOut.GetDataList().size() == 0 )   return    0;
  if( ListOut.GetDataList().back().Data.size() == 0 )   return    0;
  return    ListOut.GetDataList().back().Data.back().size();
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// ParseFrame::MoveOutListTo
///////////////////////////////////////////////////////////////////////////////////////////////////

void    ParseFrame::MoveOutListTo
        (
          list<OutputRec>::size_type    NewPos,
          vector<string>::size_type     NewPos2,
          string::size_type             NewPos2S
        )
{
  if( this->ListOut.GetDataList().size() != NewPos )
  {
    this->ListOut.GetDataList().resize( NewPos );
  }

  if( (this->ListOut.GetDataList().size() > 0 ) &&
      (this->ListOut.GetDataList().back().Data.size() != NewPos2) )
  {
    this->ListOut.GetDataList().back().Data.resize( NewPos2 );

    if( (this->ListOut.GetDataList().back().Data.size() > 0 ) &&
        (this->ListOut.GetDataList().back().Data.back().size() != NewPos2S) )
    {
      this->ListOut.GetDataList().back().Data.back().resize( NewPos2S );
    }
  }
}
}   // End namespace 'wej_parser'


