///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  FifoReaderTool.h                                                                  //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include  "FIFO_Reader.h"

using namespace std;


template<typename DataType,typename AllocType>
class  Fifo_Reader;


template<typename DataType,typename AllocType>
class  FifoReaderTool
{
public:

  FifoReaderTool( FIFO_Reader<DataType,AllocType>& FifoParam ) :
  Fifo(FifoParam),
  RTL(FifoParam)
  {
  }

  uint32_t  GetVectorLen(void) const
  {
    return  this->Fifo.GetBufMap().Dimensions.size();
  }

  FIFO_Reader<DataType,AllocType>&  GetFifo(void)
  {
    return  this->Fifo;
  }

  typename FIFO_Reader<DataType,AllocType>::ReadLockTool&  GetReadLockTool(void)
  {
    return  this->RTL;
  }

  const typename FIFO_Reader<DataType,AllocType>::ReadLockTool::ReadSchemaType&  GetReadSchema_Calc(void)
  {
    // Assemble the Write Schema and store locally
    this->RTL.GetReadSchema(this->ReadSchema);

    return  this->GetReadSchema_View();
  }

  const typename FIFO_Reader<DataType,AllocType>::ReadLockTool::ReadSchemaType&  GetReadSchema_View(void) const
  {
    return  this->ReadSchema;
  }

  uint32_t  GetRequestedReadLength(void) const
  {
    return  this->ReadSchema.RequestedReadLength;
  }

  uint32_t  GetActualReadLength(void) const
  {
    return  this->ReadSchema.ActualReadLength;
  }

private:

  FIFO_Reader<DataType,AllocType>&                               Fifo;
  typename FIFO_Reader<DataType,AllocType>::ReadLockTool                  RTL;
  typename FIFO_Reader<DataType,AllocType>::ReadLockTool::ReadSchemaType  ReadSchema;
};


