///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  TemplateRenderer.h                                                                //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include    <string>
#include    "GrammarAbs.h"
#include    "Outputs.h"
#include    "DataNodeRef.h"


class TemplateRenderer
{
private:

  static wej_parser::GrammarAbs*  GrammarConverter;
  wej_parser::Outputs             Outputs;

  /////////////////////////
  // StackRec
  /////////////////////////

  class   StackRec
  {
  private:

    DataNodeRef                             DNR;
    list<wej_parser::OutputRec>::iterator   Out;

  public:

    StackRec( const StackRec& _StackRec ) :
    DNR(_StackRec.DNR),
    Out(_StackRec.Out)
    {
    }

    StackRec( const DataNodeRef& _DI, list<wej_parser::OutputRec>::iterator _OutIter ) :
    DNR(_DI),
    Out(_OutIter)
    {
    }

    StackRec( const DataPath& _DI, list<wej_parser::OutputRec>::iterator _OutIter ) :
    DNR(DataPath(_DI),false),
    Out(_OutIter)
    {
    }

    const StackRec&   operator=( const StackRec& _StackRec )
    {
      DNR = _StackRec.GetDNR();
      Out = _StackRec.GetOut();

      return    *this;
    }

    ///////////////////////////
    // Outputs Items
    ///////////////////////////

    void    SetOut( list<wej_parser::OutputRec>::iterator _OutIter )
    {
      Out  = _OutIter;
    }

    list<wej_parser::OutputRec>::iterator   GetOut(void) const
    {
      return    Out;
    }

    ///////////////////////////
    // Children Items
    ///////////////////////////

    DataNodeRef&   GetDNR(void)
    {
      return  DNR;
    }

    const DataNodeRef&   GetDNR(void) const
    {
      return  DNR;
    }

    void  SetDNR( const DataNodeRef&  _DNR )
    {
      DNR  = _DNR;
    }

    ///////////////////////////
    // Operations
    ///////////////////////////

    void        BumpOutput(void)
    {
      ++Out;
    }

    ///////////////////////////
    // Display
    ///////////////////////////

    string      Display(void) const
    {
      ostringstream   oS;

      oS  << "DNR: " << GetDNR().Display();
      oS  << "Out: " << GetOut()->Display();

      return  oS.str();
    }
  };

  std::list<StackRec>   IterStack;

  // Output tokens on list after parse
  enum    eTokens
  {
    ORIGINAL_Token,
    BTW_Token,
    VAL_Token,
    BAR_VAL_Token,
    ITER_Token,
    END_ITER_Token,
    STR_Token
  };

  static const map<string,int>  TokenToRuleMap;

  void    DisplayRec(void)
  {
    cout << "  Size: " << IterStack.size() << endl
         << IterStack.back().Display()
         << endl;
  }

private:

  string          CurrentString;

  bool            ProcessNextStep( ostringstream& Out, const DataPath& XmlProcRoot );

  std::string     LookUpWithIndirection
                  (
                    const DataPath&     _XML_Path,
                    const DataPath&     _RootPath,
                    bool                _NoIndirection=false
                  );

public:

  TemplateRenderer(void);

  bool      Parse( istream& In );

  string    ProcessOutput( const DataPath& XmlProcRoot );
};


