///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//  FilePath:  XML_Node.h                                                                        //
//  Author:    William Earl Jones                                                                //
//  Email:     wejc@wejc.com                                                                     //
//  Tag:       DEVELOP                                                                           //
//  TimeDate:  20231110_143153                                                                   //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                               //
//                           Copyright (c) 2023 William Earl Jones                               //
//                                                                                               //
//  Redistribution and use in source and binary forms, with or without modification, are         //
//  permitted provided that the following conditions are met:                                    //
//                                                                                               //
//    1. Redistributions of source code must retain the above copyright notice, this list of     //
//       conditions and the following disclaimer.                                                //
//                                                                                               //
//    2. Redistributions in binary form must reproduce the above copyright notice, this list of  //
//       conditions and the following disclaimer in the documentation and/or other materials     //
//       provided with the distribution.                                                         //
//                                                                                               //
//    3. Neither the name of the copyright holder nor the names of its contributors may be used  //
//       to endorse or promote products derived from this software without specific prior        //
//       written permission.                                                                     //
//                                                                                               //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS  //
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF              //
//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE   //
//  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    //
//  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF           //
//  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)       //
//  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR     //
//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, //
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                           //
//                                                                                               //
///////////////////////////////////////////////////////////////////////////////////////////<wejc>//

#pragma once

#include    <list>
#include    <sstream>
#include    <iostream>
#include    <iomanip>

using namespace std;


namespace   wej_parser
{
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
// XML_Node
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class     XML_Node
{
public:

  class   AttributeRec
  {
  public:

    string    Label, Value;

  public:

    inline AttributeRec( const string LabelParam, const string ValueParam )
    {
      Label   = LabelParam;
      Value   = ValueParam;
    }
  };

private:

  string                Label;
  list<AttributeRec>    Attributes;
  list<XML_Node>        Children;
  string                Data;
  XML_Node*             Father;

public:

  inline XML_Node( XML_Node* FatherParam ) : Father(FatherParam)
  {
  }

  inline XML_Node*    GetFather(void)
  {
    return    Father;
  }

  inline void         Clear(void)
  {
    Attributes.clear();
    Children.clear();
    Label.clear();
    Data.clear();
  }

  string              WriteXML( const unsigned int Indent, const bool Compact=false ) const;

  static string       ToGlyphs( const string& In );

  /////////////////////////////
  // LookUp Numeric Data
  /////////////////////////////

  int                   GetDataAsInt(void) const;
  unsigned int          GetDataAsUnsignedInt(void) const;
  long int              GetDataAsLongInt(void) const;
  unsigned long int     GetDataAsUnsignedLongInt(void) const;
  double                GetDataAsDouble(void) const;
  float                 GetDataAsFloat(void) const;
  bool                  GetDataAsBool(void) const;

  /////////////////////////////
  // LookUp Node
  /////////////////////////////

  XML_Node*           LookUpNodeWithException( const string XmlPath );
  const XML_Node*     LookUpNodeWithException( const string XmlPath ) const;

  XML_Node*           LookUpNode( const string XmlPath );
  const XML_Node*     LookUpNode( const string XmlPath ) const;

  /////////////////////////////
  // GetXM_Path
  /////////////////////////////

  string      GetXML_Path(void) const;

  /////////////////////////////
  // Push a new attribute
  /////////////////////////////

  inline void       PushAttribute( const string LabelParam, const string ValueParam )
  {
    this->Attributes.push_back( AttributeRec( LabelParam, ValueParam ) );
  }

  // Push a new child
  inline void       PushChild( const XML_Node& Node )
  {
    this->Children.push_back( Node );
  }

  inline void       SetLabel( const string LabelParam )
  {
    Label   = LabelParam;
  }

  inline string     GetLabel(void) const
  {
    return    Label;
  }

  inline string     GetData(void) const
  {
    return    Data;
  }

  /////////////////////////////
  // Set Data
  /////////////////////////////

  inline void       SetData( const string DataParam )
  {
    Data   = DataParam;
  }

  inline void       SetData( const char* DataParam )
  {
    Data   = DataParam;
  }

  inline void       SetData( const int DataParam )
  {
    ostringstream   oS;
    oS << DataParam;
    Data   = oS.str();
  }

  inline void       SetData( const unsigned int DataParam )
  {
    ostringstream   oS;
    oS << DataParam;
    Data   = oS.str();
  }

  inline void       SetData( const long int DataParam )
  {
    ostringstream   oS;
    oS << DataParam;
    Data   = oS.str();
  }

  inline void       SetData( const unsigned long int DataParam )
  {
    ostringstream   oS;
    oS << DataParam;
    Data   = oS.str();
  }

  inline void       SetData( const float DataParam )
  {
    ostringstream   oS;
    oS << DataParam;
    Data   = oS.str();
  }

  inline void       SetData( const double DataParam )
  {
    ostringstream   oS;
    oS << DataParam;
    Data   = oS.str();
  }

  inline void       SetData( const bool DataParam )
  {
    ostringstream   oS;
    oS << DataParam;
    Data   = oS.str();
  }

  /////////////////////////////
  // Attributes Browsing
  /////////////////////////////

  typedef list<XML_Node::AttributeRec>::iterator          AttributesIterator;
  typedef list<XML_Node::AttributeRec>::const_iterator    AttributesConstIterator;

  list<XML_Node::AttributeRec>::size_type                 GetNumAttributes(void) const
  {
    return    Attributes.size();
  }

  inline list<XML_Node::AttributeRec>::iterator           AttributesBegin(void)
  {
    return    Attributes.begin();
  }

  inline list<XML_Node::AttributeRec>::iterator           AttributesEnd(void)
  {
    return    Attributes.end();
  }

  inline list<XML_Node::AttributeRec>::const_iterator     AttributesBegin(void) const
  {
    return    Attributes.begin();
  }

  inline list<XML_Node::AttributeRec>::const_iterator     AttributesEnd(void) const
  {
    return    Attributes.end();
  }

  /////////////////////////////
  // Children Browsing
  /////////////////////////////

  typedef list<XML_Node>::iterator            ChildrenIterator;
  typedef list<XML_Node>::const_iterator      ChildrenConstIterator;

  bool    IsLeaf(void) const
  {
    return  Children.size() == 0;
  }

  list<XML_Node>&                             GetChildrenList(void)
  {
    return    Children;
  }

  const list<XML_Node>&                       GetChildrenList(void) const
  {
    return    Children;
  }

  list<XML_Node>::size_type                   GetNumChildren(void) const
  {
    return    Children.size();
  }

  inline list<XML_Node>::iterator             ChildrenBegin(void)
  {
    return    Children.begin();
  }

  inline list<XML_Node>::iterator             ChildrenEnd(void)
  {
    return    Children.end();
  }

  inline XML_Node&                            ChildrenBack(void)
  {
    return    Children.back();
  }

  inline XML_Node&                            ChildrenFront(void)
  {
    return    Children.front();
  }

  inline list<XML_Node>::const_iterator       ChildrenBegin(void) const
  {
    return    Children.begin();
  }

  inline list<XML_Node>::const_iterator       ChildrenEnd(void) const
  {
    return    Children.end();
  }

  inline const XML_Node&                      ChildrenBack(void) const
  {
    return    Children.back();
  }

  inline const XML_Node&                      ChildrenFront(void) const
  {
    return    Children.front();
  }

  /////////////////////////////
  // Display
  /////////////////////////////

  string    Display( const unsigned int Indent = 0 ) const;

};
}   // End namespace 'wej_parser'


